# Generic alter procedure
# Date:     08/10/97
# Author:   Robin Sergeant

proc g_alter { gcl_object instvars } {
    global g_colour_options g_colour_label g_colour_gcl g_line_options 
    global g_line_label g_line_gcl g_font_options g_font_label g_font_gcl
    global g_symbol_options g_symbol_label g_symbol_gcl
    g_winPosition .alter
    switch $gcl_object {
        noObject {wm title .alter "Graphics Defaults"}
        noObject2 {wm title .alter "Program Defaults"}
        default {wm title .alter "Alter Object"}
    }
    array set instarray $instvars
    set keys [array names instarray]
    foreach inst [lsort $keys] {        
        set val $instarray($inst)
        switch -glob $inst {
            *colour {
               g_gclOptions $inst colour
            }
            lineThickness {
                g_labelEntry "Line Thickness"
            }
            type {
                g_gclOptions "Line Type" line
            }
            binCentre {
                g_radioButtons notCentred centred
            }
            mode {
                g_radioButtons "mode:histogram" "mode:point"
            }
            quality {
                g_gclOptions "Text Font" font
            }
            charsize {
                g_labelEntry "Text Size"
            }
            markersize {
                g_labelEntry "Marker Size"
            }
            xlog {
                g_radioButtons xlinear xlog
            }
            ylog {
                g_radioButtons ylinear ylog
            }
            xnum {
                g_radioButtons noxnum xnum
            }
            ynum {
                g_radioButtons noynum ynum
            }
            yhoriz {
                g_radioButtons yvertical yhorizontal
            }
            vertical {
                g_radioButtons horizontal vertical
            }
            words {
                g_labelEntry "Text"
            }
            xlabel {
                g_labelEntry "Xlabel"
            }
            ylabel {
                g_labelEntry "Ylabel"
            }
            solid {
                g_radioButtons outline filled
            }
            symbol {
                g_gclOptions "Symbol" symbol
            }
            ncont {
                g_labelEntry "No. of contours"
            }
            itype {
                g_radioButtons linear log sqrt
            }
            xvert {
                g_radioButtons xhorizontal xvertical
            }
            cellType {
                g_radioButtons raw smooth
            }
            valMin {
                g_labelEntry "Cut off min."
            }
            valMax {
                g_labelEntry "Cut off max."
            }
            nHist {
                g_labelEntry "No. of histograms"
            }
            chmin {
                g_labelEntry "X data min. cut off"
            }
            chmax {
                g_labelEntry "X data max. cut off"
            }
            angle {
                g_labelEntry "Text angle"
            }
            start {
                g_labelEntry "Start value"
            }
            end {
                g_labelEntry "End value"
            }
            step {
                g_labelEntry "Step"
            }
            intval {
                g_labelEntry "Label interval"
            }
            minint {
                g_labelEntry "Min. no. of cells"
            }
            table {
                set val Unchanged
                g_options "Colour Table" "Unchanged Heat Rainbow"
            }
            graticule {
                g_checkButton "Draw Graticule"
            }
           	hardcopy {
                g_options "Hardcopy" "PS CPS CGM CGMD"
            }
            ygap {
                g_labelEntry "YGAP"
            }
            ?rounding {
                g_checkButton $inst
            }
            template_dir {
                g_labelEntry "User template directory"
            }
            working_dir {
                g_labelEntry "Working directory"
            }
            data_dir {
                g_labelEntry "Data file directory"
            }
            ext_dir {
                g_labelEntry "Tcl extensions directory"
            }
            gcl_dir {
                g_labelEntry "Gcl code directory"
            }
            gmod_dir {
                g_labelEntry "Genie modules directory"
            }                        
            show_tips {
                g_checkButton "Enable tips"
            }
            binning {
                g_labelEntry "Histogram binning"
            }
            mouse_button {
                g_options "Mouse selection button" "<Button-2> <Button-3>"
            }            
        }
    }
    tixButtonBox .alter.box -orientation horizontal
    .alter.box add ok -text Ok -command "g_alterOK { $gcl_object { $instvars } };\
        catch {destroy .alter}"
    if {[string match noObject* $gcl_object] != 1} {
        .alter.box add delete -text Remove -command "g_remove { $gcl_object };\
            catch {destroy .alter}"
    } 
    .alter.box add cancel -text Cancel -command "catch {destroy .alter}"
    pack .alter.box -side top -expand yes -fill x
    bind .alter <Return> ".alter.box.ok invoke"
}

proc g_labelEntry { label } {
# Displays a generic tixLabelEntry widget.
# The widget is given the name of the current instance variable (eg. colour).
# A variable of the same name as the instance variable is assigned to the
# widget.  It is then assigned the value of the instance variable (eg. $red).
    upvar inst inst
    upvar val val
    tixLabelEntry .alter.$inst -label $label -labelside left
    .alter.$inst subwidget entry config -textvariable gw_$inst
    .alter.$inst subwidget entry delete 0 end
    .alter.$inst subwidget entry insert 0 $val
    pack .alter.$inst -side top -expand yes -anchor e
}

proc g_radioButtons { args } {
# Displays a generic set of radio buttons in the same manner as g_labelEntry
    upvar inst inst
    upvar val val
    upvar gcl_object gcl_object    
    frame .alter.$inst
    pack .alter.$inst
    foreach type $args {
        radiobutton .alter.$inst.$type -text $type -value $type -variable gw_$inst
        pack .alter.$inst.$type -side left -expand yes
    }
    if {$gcl_object != "noObject"} {
        .alter.$inst.[lindex $args $val] select
    } else {
        .alter.$inst.$val select
    }
}

proc g_checkButton { label } {
# Displays a generic check button in the same manner as g_labelEntry
    upvar inst inst
    upvar val val
    checkbutton .alter.$inst -text $label -variable gw_$inst
    if {$val == 1} {
        .alter.$inst select
    }
    pack .alter.$inst -side top -expand yes
}

proc g_gclOptions { label name } {
# Displays a generic options button in the same manner as g_labelEntry.
# The arrays needed are accessed by building up their names from the name
# parameter and using upvar to allow access.
    upvar inst inst
    upvar val val
    upvar gcl_object gcl_object
    set end _gcl
    upvar g_$name$end gcl
    set end _options
    upvar g_$name$end options
    set end _label
    upvar g_$name$end glabel
    if {$gcl_object != "noObject"} {
        set val $gcl($val)
    }
    tixOptionMenu .alter.$inst -label $label -variable gw_$inst
    foreach opt $options {
        .alter.$inst add command $opt -label $glabel($opt)
    }
    .alter.$inst config -value $val
    pack .alter.$inst -side top -expand yes -anchor e
}

proc g_options { label options } {
# Displays a generic options button in the same manner as g_labelEntry
    upvar inst inst
    upvar val val
    tixOptionMenu .alter.$inst -label $label -variable gw_$inst
    foreach opt $options {
        .alter.$inst add command $opt -label $opt
    }
    .alter.$inst config -value $val
    pack .alter.$inst -side top -expand yes -anchor e
}

proc g_alterOK { args } {
    set args [lindex $args 0]
    set gcl_object [lindex $args 0]
    set instvars [lindex $args 1]
    set gcl_command "gobject"
    set gcl_parameters " object=$gcl_object "
    set gcl_qualifiers "/alter"
    set len [llength $instvars]
    for {set c 0} {$c < $len} {incr c 2} {
        set inst [lindex $instvars $c]
        upvar gw_$inst val
          switch -glob $inst {
            *colour {
                g_append $gcl_object gcl_parameters colour= $inst $val
            }
            lineThickness {
                set val [string trim $val]
                if {[string len $val] > 0} {
                    g_append $gcl_object gcl_parameters line_thickness= $inst $val
                }
            }
            type {
                g_append $gcl_object gcl_parameters line_type= $inst $val
            }
            binCentre {
                if {$val == "centred"} {
                    set val "bincentre"
                }
                append gcl_qualifiers "/$val"
            }
            quality {
                g_append $gcl_object gcl_parameters font= $inst $val
            }
            *size {
                set val [string trim $val]
                if {[string len $val] > 0} {
                    g_append $gcl_object gcl_parameters size= $inst $val
                }
            }
            xlog {
                g_append $gcl_object gcl_qualifiers / $inst $val
            }
            ylog {
                g_append $gcl_object gcl_qualifiers / $inst $val
            }
            xnum {
                append gcl_qualifiers "/$val"
            }
            ynum {
                append gcl_qualifiers "/$val"
            }
            yhoriz {
                append gcl_qualifiers "/$val"
            }
            vertical {
                append gcl_qualifiers "/$val"
            }
            words {
                set val [string trim $val]
                append gcl_parameters "text=\"$val\" "
            }
            xlabel {
                set g_val [string trim $val]
                append gcl_parameters "xlabel=\"$val\" "
            }
            ylabel {
                set g_val [string trim $val]
                append gcl_parameters "ylabel=\"$val\" "
            }
            solid {
                append gcl_qualifiers "/$val"
            }
            symbol {
                g_append $gcl_object gcl_parameters symbol= $inst $val
            }
            ncont {
                set val [string trim $val]
                if {[string len $val] > 0} {
                    append gcl_parameters "ncont=$val "
                }
            }
            itype {
                append gcl_qualifiers "/$val"
            }
            xvert {
                append gcl_qualifiers "/$val"
            }
            cellType {
                append gcl_qualifiers "/$val"
            }
            valMin {
                set val [string trim $val]
                if {[string len $val] > 0} {
                    append gcl_parameters "valmin=$val "
                }
            }
            valMax {
                set val [string trim $val]
                if {[string len $val] > 0} {
                    append gcl_parameters "valmax=$val "
                }
            }
            nHist {
                set val [string trim $val]
                if {[string len $val] > 0} {
                    append gcl_parameters "numhist=$val "
                }
            }
            chmin {
                set val [string trim $val]
                if {[string len $val] > 0} {
                    append gcl_parameters "chmin=$val "
                }
            }
            chmax {
                set val [string trim $val]
                if {[string len $val] > 0} {
                    append gcl_parameters "chmax=$val "
                }
            }
            angle {
                set val [string trim $val]
                if {[string len $val] > 0} {
                    append gcl_parameters "angle=$val "
                }
            }
            start {
                set val [string trim $val]
                if {[string len $val] > 0} {
                    append gcl_parameters "start=$val "
                }
            }
            end {
                set val [string trim $val]
                if {[string len $val] > 0} {
                    append gcl_parameters "end=$val "
                }
            }
            step {
                set val [string trim $val]
                if {[string len $val] > 0} {
                    append gcl_parameters "st=$val "
                }
            }
            intval {
                set val [string trim $val]
                if {[string len $val] > 0} {
                    append gcl_parameters "intval=$val "
                }
            }
            minint {
                set val [string trim $val]
                if {[string len $val] > 0} {
                    append gcl_parameters "minint=$val "
                }
            }
            table {
                if {$val != "Unchanged"} {
                    set oldval [lindex $instvars [expr $c + 1]]
                    append gcl_parameters "table=colourtable:$val\($oldval\) "
                }
            }
            graticule {
                g_append $gcl_object _ _ $inst $val
            }
            hardcopy {
                g_append $gcl_object _ _ $inst $val
            }
            ygap {
                g_append $gcl_object _ _ $inst $val
            }
            ?rounding {
                g_append $gcl_object _ _ $inst $val
            }
            template_dir {
                g_append $gcl_object _ _ $inst $val
            }
            working_dir {
                g_append $gcl_object _ _ $inst $val
            }
            data_dir {
                g_append $gcl_object _ _ $inst $val
            }
            ext_dir {
                g_append $gcl_object _ _ $inst $val
            }
            gcl_dir {
                g_append $gcl_object _ _ $inst $val
            }
            gmod_dir {
                g_append $gcl_object _ _ $inst $val
            }                                    
            show_tips {
                g_append $gcl_object _ _ $inst $val
            }
            binning {
                g_append $gcl_object _ _ $inst $val
            }
            mode {
                g_append $gcl_object _ _ $inst $val
            }
            mouse_button {
                g_append $gcl_object _ _ $inst $val
            }
        }
    }
    if {[string match noObject* $gcl_object] != 1} {
        append gcl_command $gcl_qualifiers
        append gcl_command $gcl_parameters
        puts $gcl_command
        set output [gcl $gcl_command]
        g_testOutput $output
    }
}

proc g_append { gcl_object list string inst val } {
    global g_gd g_pd g_defaultsModify
    switch $gcl_object {
        noObject {
			if {$g_gd($inst) != $val} {
				set g_defaultsModify 1
            	set g_gd($inst) $val
			}
        }
        noObject2 {
			if {$g_pd($inst) != $val} {
				set g_defaultsModify 1
            	set g_pd($inst) $val
			}
        }
        default {
            upvar $list realList
            append realList "$string$val "
        }
    }    
}

proc g_remove { gcl_object } {
    if {[tk_dialog .dialog "Conformation" "Are you sure?" "" 1 Yes No] == 0} {
        set output [gcl "delete $gcl_object"]
        g_testOutput $output
    }    
}

proc g_winPosition {win} {
# called after all top level windows are created to control their position
# win should contain the pathname of the newly created window
# get the position of the main parent window
    catch {destroy $win}
    toplevel $win
    set parent [winfo parent $win]
    set x [winfo x $parent]
    set y [winfo y $parent]
# set the location of the new window to be slightly adrift
    set x [expr $x + 20]
    set y [expr $y + 20]
# move the new window to x,y
    wm geometry $win +$x+$y
# make window a child of main, so that it is tied to the main aplication
    wm transient $win $parent
}



    
