###########################################################################
# OPEN Genie Tk/Tcl GUI                                                   #
# Date:     17/10/97 - 03/07/98                                           #
# Author:   Robin Sergeant                                                #
###########################################################################

###########################################################################
#General comments:
# - data is stored in sets, each set maps to an element in a genie array of
#   workspaces.
# - inorder to work with data the program must keep track of these sets.
#   To do this it stores information about each set, including which fields 
#   to use for x, y, e data, as well as where the data came from.
# - wherever possible a defaults array is used to supply parameters for genie
#   commands.
# - the interface consists of both pull down menus and a button bar for the
#   more important operations.  If any of these operations are not appropriate
#   then they are greyed out and disabled.
###########################################################################

# get any startup information printed to GENIE streams
set rubbish [gcl "print"]

# test for any startup errors
g_testOutput "$rubbish"	

# where we get "genie on the net" from 
set g_wwwserver ndadixwell.nd.rl.ac.uk

# CGI WWW server - used for mailing list subscription etc.
set g_cgiserver isolde.nd.rl.ac.uk

# prefix to apply to web names - for ndadixwell this will usually be
# "-prepub", but for the final release on www.isis.rl.ac.uk it will
# be blank
set g_wwwsuffix -prepub

option add *Entry.width 20

# g_lastFocus is used for pasting from the clipboard
set g_lastFocus .
bind Entry <Delete> {%W delete [expr [%W index insert]-1]}
bind Entry <Control-u> {%W delete 0 insert}
bind Entry <Control-U> {%W delete 0 insert}
bind Entry <Control-a> {%W icursor 0}
bind Entry <Control-A> {%W icursor 0}
bind Entry <Control-e> {%W icursor end}
bind Entry <Control-E> {%W icursor end}
bind Entry <FocusOut> {set g_lastFocus %W}
bind Entry <Control-x> g_cut
bind Entry <Control-X> g_cut
bind Entry <Control-c> g_copy
bind Entry <Control-C> g_copy
bind Entry <Control-v> g_paste
bind Entry <Control-V> g_paste
bind Text <Delete> {%W delete "insert -1 char"}
bind Text <Control-a> {%W tag add sel 0.0 end}
bind Text <Control-A> {%W tag add sel 0.0 end}
bind Text <FocusOut> {set g_lastFocus %W}
bind Text <Control-x> g_cut
bind Text <Control-X> g_cut
bind Text <Control-c> g_copy
bind Text <Control-C> g_copy
bind Text <Control-v> g_paste
bind Text <Control-V> g_paste

proc g_deactivate {args} {
# deactivate or grey out one or more widgets
# args should contain a list of widget paths, eg. .button1 etc.
    foreach b $args {
        $b config -state disabled
     }
}

proc g_Mdeactivate {path args} {
# deactivate or grey out one or more items on a menu
# path should contain the path of the menu widget, eg. .menu1
# args should contain a list of the menu item numbers, eg. 0 3 4 etc.
    foreach index $args {
        $path entryconfigure $index -state disabled 
    }
}

proc g_activate {args} {
# activate one or more widgets
    foreach b $args {
        $b config -state normal
    }
}

proc g_Mactivate {path args} {
# activate one or more items on a menu
    foreach index $args {
        $path entryconfigure $index -state normal
    }
}

proc g_changeCursor {new win} {
    foreach w [winfo children $win] {
        catch {$w config -cursor $new}
        g_changeCursor $new $w
    }
    update idletasks
}

proc g_resetCursor {win} {
    foreach w [winfo children $win] {
        set class [winfo class $w]
        if {($class == "Text") | ($class == "Entry")} {
            $w config -cursor xterm
        } else {
            catch {$w config -cursor ""}
        }
        g_resetCursor $w
    }    
    update idletasks
}

proc g_tip {} {
    global g_tkDir g_pd;
    set t .tip
    g_winPosition $t
    wm title $t "Did you know?"
    set fileid [open $g_tkDir/tips.dat {RDONLY}]
    set n [gets $fileid]
    set rnd [smalltalk "| r | r _ Random new.(r next * $n) truncated prn!" -out]
    set tipText ""
    for {set count 0} {$count <= $rnd} {incr count} {
        set tipText [gets $fileid]
    }
    close $fileid
    message $t.text -width 200 -text $tipText
    checkbutton $t.check -text "show next time" -variable gw_nomore
    $t.check select
    button $t.ok -text Ok -command "set g_pd(show_tips) \$gw_nomore; \
        g_saveDefaults 1; catch {destroy $t}"
    pack $t.text $t.check $t.ok -expand yes
    bind $t <Return> "$t.ok invoke"
}

proc g_splash {} {
    global g_tkDir
    set im [image create photo -file $g_tkDir/splash.gif]
    set text [gcl "warranty"]
    set text [lindex $text 2]
    set o .output
    g_winPosition $o
    wm title $o "Welcome to Open Genie"
    $o config -bg white
    label $o.gif -image $im
    text $o.main -height 20 -width 70 -bg white -relief flat -borderwidth 0
    button $o.button -text Ok -command "catch {destroy $o}" -bg white
    pack $o.gif
    pack $o.main -fill both -expand true
    $o.main insert end $text
    $o.main config -state disabled
    g_changeCursor watch .
    tkwait visibility $o.main
    after 3000 "catch {destroy $o; g_resetCursor .}"
}

# setup genie workspace array to store up to 100 sets + temp
gcl "wa=dimensions(101);count=0"
# set the current set number to 0, indicating no data
set g_current 0
#initialise genie current set string to be undefined
set g_current_set _
# set the current filename to a dummy value
set g_filename ./.
# initialise the set details array
set g_set(1,wk) 0
# initialise the module arrays
set g_Imodules(none) _
set g_Tmodules(none) _
set g_Mlang(none) _ 
set g_new_search 0
set g_search_value ""
# put a trace on this array, so that g_setinfo is called if it changes
# NB g_setinfo is used to update the set status line.
trace variable g_set w g_setinfo
# keep g_current_set up to date
trace variable g_current w g_setTrace
# set the clear flag to 1, so that the display is cleared between plots
set gw_clear 1
# set limits
set g_limits(xmin) 0.0
set g_limits(xmax) 1.0
set g_limits(ymin) 0.0
set g_limits(ymax) 1.0
# include a 0.2 border
set g_limits(dxmin) 0.2
set g_limits(dxmax) 0.8
set g_limits(dymin) 0.2
set g_limits(dymax) 0.8
# note weather limis need to be loged
set g_limits(xminlog) 0
set g_limits(xmaxlog) 0
set g_limits(yminlog) 0
set g_limits(ymaxlog) 0
# defaults flag (1 if altered but not saved)
set g_defaultsModify 0

# setup the graphics defaults array
# NB not all values are currently used

# OS specific definitions
set g_os [gcl "print os()" -out]
if { [ string compare $g_os "WINNT" ] == 0 } {
    set g_homeDir $env(USERPROFILE)
    set g_directorySeparator "\\"
    load "$env(TCL_LIBRARY)/../tix4180.dll" Tix
} elseif { [ string compare $g_os "WIN9X" ] == 0 } {
    set g_homeDir "\\"
    set g_directorySeparator "\\"
    load "$env(TCL_LIBRARY)/../tix4180.dll" Tix
} elseif { [ string compare $g_os "VMS" ] == 0 } {
    set g_homeDir $env(HOME)
    set g_directorySeparator ""
} else {
    set g_homeDir $env(HOME)
    set g_directorySeparator "/"
}

set g_defaultsFile "${g_homeDir}${g_directorySeparator}genieTCLDefaults.dat"

# This is a fiddle for VMS as it doesn't export GENIE_TCL_DIR
set genie_tcl_dir [ array names env GENIE_TCL_DIR ]
if { [ string compare "$genie_tcl_dir" "GENIE_TCL_DIR" ] != 0 } { 
    set g_tkDir $env(TCL_LIBRARY)/.. 
} else {
    set g_tkDir $env(GENIE_TCL_DIR)
}

# load in TCL versions of GCL commands
set rubbish [gcl "_SDJ_GW.INTERACTIVE=\"TK\""]
g_testOutput "$rubbish"	
set rubbish [smalltalk "Device defaultDevType: 'tk'!"]
g_testOutput "$rubbish"	
set rubbish [gcl "PROCEDURE INQUIRE;PARAMETERS i=string;RESULT res;res=tk_inquire(i);ENDPROCEDURE"]
g_testOutput "$rubbish"	
set rubbish [gcl "PROCEDURE READ_TERMINAL;RESULT res;res=tk_read_terminal();ENDPROCEDURE"]
g_testOutput "$rubbish"	
set rubbish [gcl "\$TK_INTERFACE=1"]
g_testOutput "$rubbish"	

set g_docDir [gcl {print _getenv("GENIE_DOC_DIR")} -out]

set g_template(Default) $g_tkDir/Templates/blank.gtp

set pwd [pwd]
cd $g_tkDir/Templates
foreach fname [glob *.gtp] {
    set fileid [open $fname r] 
    set tname [gets $fileid]
    close $fileid
    set g_template($tname) $g_tkDir/Templates/$fname
}
cd $pwd
    
set g_gd(xlog) xlinear
set g_gd(ylog) ylinear
set g_gd(hardcopy) CPS
#set g_gd(ygap) 10.0
set g_gd(graticule) 0
set g_gd(markersize) 0.7
set g_gd(lineThickness) 1.0
set g_gd(charsize) 1.5
set g_gd(quality) \$Normal
set g_gd(symbol) \$Plus
set g_gd(type) \$Full
set g_gd(line_colour) \$Black
set g_gd(text_colour) \$Blue
set g_gd(plot_colour) \$Red
#set g_gd(xrounding) 1
#set g_gd(yrounding) 1   
set g_gd(binning) 1
set g_gd(mode) "mode:histogram"
set g_pd(template_dir) $g_homeDir
set g_pd(working_dir) $g_homeDir
set g_pd(data_dir) ${g_tkDir}${g_directorySeparator}..${g_directorySeparator}examples${g_directorySeparator}data
set g_pd(ext_dir) $g_homeDir
set g_pd(gcl_dir) $g_homeDir
set g_pd(gmod_dir) $g_homeDir
set g_pd(show_tips) 1
# g_pd(mouse_button) also read by tkdriv.c
set g_pd(mouse_button) <Button-3>	
set g_pd(devwidth) 500
set g_pd(devheight) 500
set g_pd(scratch) "printn \"Welcome to Open Genie....\""
set g_pd(name) NONE
set g_pd(email) NONE
set g_pd(printer) NONE

if {[file exists $g_defaultsFile]} {
    set fileid [open $g_defaultsFile r]
    array set g_gd [gets $fileid]
	array set g_pd [gets $fileid]
    close $fileid
# convert dummy characters in g_pd(scratch) to newline characters before saving
#set newline []
	set temp [format "%c" 6]
	regsub -all $temp $g_pd(scratch) "\n" g_pd(scratch)
}

proc g_update_gcl_defaults {} {
    global g_gd
    gcl "alter/linecolour $g_gd(line_colour)"
    gcl "alter/plotcolour $g_gd(plot_colour)"
    gcl "alter/textcolour $g_gd(text_colour)"
    gcl "alter/linetype $g_gd(type)"
    gcl "alter/markers $g_gd(symbol)"
    gcl "alter/markersize $g_gd(markersize)"
    gcl "alter/font $g_gd(quality)"
    gcl "alter/linewidth $g_gd(lineThickness)"
    gcl "alter/textheight $g_gd(charsize)"
}

set g_fieldNames(x) "X Array"
set g_fieldNames(y) "Y Array"
set g_fieldNames(e) "E Array"
set g_fieldNames(t) "Plot Title"
set g_fieldNames(xl) "X Axis Label"
set g_fieldNames(yl) "Y Axis Label"
foreach name [array names g_fieldNames] {
    set id $g_fieldNames($name)
    set g_fieldIDs($id) $name
}

# allow for more than one Genie window
wm title . [tk appname "Open Genie"]
wm geometry . +25+25
wm minsize . 100 100

frame .m
menubutton .m.file -text File -menu .m.file.f -underline 0
set m .m.file.f
menu $m -tearoff 0
$m add cascade -label "Read" -menu $m.read -underline 0
$m add cascade -label "Write" -menu $m.write -underline 0
$m add command -label "Open Genie Console" -command g_console -underline 11
$m add command -label "Load tcl extension" -command "g_read_data tcl" \
    -underline 5
$m add command -label "Load gcl code" -command "g_read_data gcl" -underline 5
$m add command -label "Load genie module" -command "g_read_data gmod" \
    -underline 11
$m add command -label "Print" -command g_hardcopy -underline 0
$m add command -label "Exit" -command g_exit -underline 1 -accelerator \
    Ctrl+Z
bind . <Control-z> g_exit
bind . <Control-Z> g_exit
set m $m.read
menu $m -tearoff 0
$m add command -label "ASCII Free Format" -command "g_read_data a1" \
    -underline 9
$m add command -label "ASCII Fixed Format" -command "g_read_data a2" \
    -underline 8
$m add command -label "Binary" -command "g_read_data b" -underline 0
if {$g_os == "VMS"} {
    $m add command -label "DAE" -command g_read_dae -underline 0
}
set m .m.file.f.write
menu $m -tearoff 0
$m add command -label "ASCII" -command "g_write_data wa" -underline 0
$m add command -label "NEXUS" -command "g_write_data wn" -underline 0
$m add command -label "Genie Binary" -command "g_write_data wgb" -underline 6
menubutton .m.edit -text Edit -menu .m.edit.e -underline 0
set m .m.edit.e
menu $m -tearoff 0
$m add command -label "Cut" -command g_cut -underline 2 -accelerator Ctrl+X
$m add command -label "Copy" -command g_copy -underline 0 -accelerator Ctrl+C
$m add command -label "Paste" -command g_paste -underline 0 -accelerator Ctrl+V
menubutton .m.select -text Select -menu .m.select.s -underline 0
set m .m.select.s
menu $m -tearoff 0
$m add command -label "Select Set" -command g_select_set -underline 7
$m add command -label "Select Data" -command g_newBrowse -underline 7
$m add command -label "Select Limits" -command g_limits -underline 7
$m add command -label "Select Picture" -command g_picture -underline 7 
menubutton .m.operations -text Operations -menu .m.operations.oper -underline 1
set m .m.operations.oper
menu $m -tearoff 0
$m add command -label "Units" -command g_units -underline 0
$m add cascade -label "Rebin" -menu $m.rebin -underline 0
$m add command -label "Arithmetic" -command g_arthmetic -underline 0
$m add command -label "Module Transformations" -command g_module_transform \
    -underline 0
set m $m.rebin
menu $m -tearoff 0
$m add command -label "Rebin one set against another" -command g_rebin \
    -underline 22
$m add command -label "Rebin one set over a range" -command g_rebin2 \
    -underline 21
$m add command -label "Rebin one set in given steps" -command g_rebin3 \
    -underline 23
menubutton .m.draw -text Draw -menu .m.draw.d -underline 0
set m .m.draw.d
menu $m -tearoff 0
$m add cascade -label Plot -menu $m.plot -underline 0
$m add cascade -label "Two dimensional plot" -menu $m.plot2d -underline 0
$m add command -label "Function plot" -command g_funcplot
$m add cascade -label "Peak fit" -menu $m.peak -underline 5
$m add cascade -label Annotate -menu $m.annotate -underline 0
set m $m.plot
menu $m -tearoff 0
$m add command -label "Plot Line Graph" -command "g_draw \$gw_clear line" \
    -underline 5
$m add command -label "Plot Histogram" -command "g_draw \$gw_clear histogram" \
    -underline 5
$m add command -label "Plot Error Bars" -command "g_draw \$gw_clear error" \
    -underline 5
$m add command -label "Plot Markers" -command "g_draw \$gw_clear marker" \
    -underline 5
set m .m.draw.d.plot2d
menu $m -tearoff 0
$m add command -label "Multiplot" -command "g_draw \$gw_clear multi"
$m add command -label "Contour plot" -command g_contour
$m add command -label "Contour array plot" -command g_contourArray
$m add command -label "Cell plot" -command g_cell
$m add command -label "Cell array plot" -command g_cellArray
$m add command -label "Surface plot" -command g_3dPlot
set m .m.draw.d.peak
menu $m -tearoff 0
$m add command -label "Gaussian" -command {g_peakfit gauss}
$m add command -label "Gaussian + exponential" -command {g_peakfit gexp}
$m add command -label "Lorentzian" -command {g_peakfit loren}
$m add command -label "Lorentzian + exponential" -command {g_peakfit lexp}
$m add command -label "Voigt" -command {g_peakfit voigt}
$m add command -label "Voigt + exponential" -command {g_peakfit vexp}
$m add command -label "Polynomial" -command g_peakfit_poly
set m .m.draw.d.annotate
menu $m -tearoff 0
$m add command -label "Add title" -command {g_text "Plot Title" \
    Title: g_title} -underline 4
$m add command -label "Add label to X axis" -command {g_text "X axis label" \
    Label: g_xlabel} -underline 13
$m add command -label "Add label to Y axis" -command {g_text "Y axis label" \
    Label: g_ylabel} -underline 13
$m add command -label "Add contour labels" -command g_contourLabels    
$m add command -label "Add text" -command {tk_dialog .text "Add text" \
    "Click the mouse selection button in the plot area!" "" 0 Ok}
menubutton .m.options -text Options -menu .m.options.o -underline 0
set m .m.options.o
menu $m -tearoff 0
$m add command -label "Graphics Defaults" -command g_graphics_defaults \
    -underline 0
$m add command -label "Program Defaults" -command g_program_defaults \
    -underline 0
$m add command -label "Save Defaults" -command {g_saveDefaults 0} -underline 0
menubutton .m.help -text Help -menu .m.help.h -underline 0
set m .m.help.h
menu $m -tearoff 0
$m add command -label "Contents" -command {g_help index.htm} -underline 0
$m add command -label "Overview" -command {g_help overview.htm} -underline 0
$m add command -label "Getting Started" -command {g_help started.htm} -underline 8
$m add command -label "Example Programs" -command "g_help index.htm \
    $g_tkDir/../examples" -underline 0
$m add cascade -label "Genie on the Web..." -menu $m.web -underline 14
$m add command -label "Warranty" -command {g_help warranty.htm} -underline 0
$m add command -label "Licence" -command {g_help gnu.htm} -underline 0
$m add command -label "About" -command g_about -underline 0
set m $m.web
menu $m -tearoff 0
$m add command -label "Frequently Asked Questions" -command g_webFAQ \
    -underline 0
$m add command -label "News" -command g_webNews -underline 0
$m add command -label "Subscribe to genie-announce mailing list" -command \
    g_webSub -underline 0
$m add command -label "Unsubsribe from genie-announce mailing list" -command \
    g_webUnsub -underline 0
$m add command -label "Report a bug" -command g_webReport -underline 10
menubutton .m.ext -text Extensions -menu .m.ext.e -underline 0
set e .m.ext.e
menu $e -tearoff 0 

frame .b
frame .b.coord -borderwidth 2 -relief ridge
label .b.coord.x -text "x:" -width 15
label .b.coord.y -text "y:" -width 15
checkbutton .b.clear -text "Clear before Plot" -variable gw_clear
tixButtonBox .b.box -orientation vertical
.b.box add plot -text "Plot Data" -command "g_plot_data \$gw_clear"
.b.box add browse -text "Browse!" -command g_newBrowse
.b.box add zoom -text "Zoom" -command g_zoom
.b.box add hardcopy -text "Hardcopy" -command g_hardcopy
.b.box add limits -text "Limits" -command g_limits

pgwindow .p1 -width $g_pd(devwidth) -height $g_pd(devheight) -borderwidth 2 \
    -relief ridge
# setup the set status line
label .set -text "Set: 0 From: _ Block/Spectrum: _"
pack .set -side bottom -fill x
pack .m -side top -fill x
pack .m.file .m.edit .m.select .m.operations .m.draw .m.ext .m.options \
    -side left
pack .m.help -side right
pack .b -side left -fill y
pack .b.box -side bottom -pady 0.25i
pack .b.clear -side bottom
pack .b.coord -side bottom -pady 0.25i
pack .b.coord.x .b.coord.y
pack .p1 -side left -expand 1 -fill both
tkwait visibility .p1
gcl "device/open \"tk\""
# Get background Tk "grey" colour
gcl "\$TK_GREY=COLOUR:NAMED(\"LightGrey\"); \$TK_GRAY=\$TK_GREY"
# create an initial unscaled window to display the axes
gcl "win=win_unscaled(0.2,0.8,0.2,0.8)"
gcl "axes/draw/$g_gd(xlog) /$g_gd(ylog) $g_gd(line_colour) $g_gd(charsize) \
    $g_gd(quality) $g_gd(lineThickness) $g_gd(type)"
# deactivate all menu items and buttons which require data inorder to function
if {$g_os == "WINNT" | $g_os == "WIN9X"} {
    g_deactivate .b.box.zoom
	font create boldTitle -family Courier -size 10 -weight bold
	font create listText -family Courier -size 10 -weight normal
}
g_deactivate .b.box.plot .b.box.browse .b.box.limits
g_Mdeactivate .m.file.f 1
g_Mdeactivate .m.select.s 0 1 2
g_Mdeactivate .m.operations.oper 0 1 2 3
g_Mdeactivate .m.draw.d 0 1 3
g_splash
# show 'tip of the day' if enabled
if {$g_pd(show_tips) == 1} {
    g_tip
}
bind .p1 <Configure> {gcl "redraw"}
# bind the choosen mouse button to the procedure g_annotate
bind .p1 $g_pd(mouse_button) {g_annotate %x %y}
#bind mouse movement to g_coordinates
bind .p1 <Motion> {g_coordinates %x %y}
wm protocol . WM_DELETE_WINDOW g_exit
tixBalloon .bal
.bal bind .b.box.plot -msg "Produce a plot from the current set"
.bal bind .b.box.browse -msg "Browse data elements and work with templates"
.bal bind .b.box.zoom -msg "Zoom in on the display area"
.bal bind .b.box.hardcopy -msg "Send a hardcopy of the display area to a \
    printer or file"
.bal bind .b.box.limits -msg "Set linear limits for the current set"

g_update_gcl_defaults

proc g_read_data { type } {
# The first procedure called when reading data into a set
# type should contain a string representing the file type:
#   b = binary, a1 = free format ascii, a2= fixed format ascii
    global g_filename g_pd g_Imodules
# a regular expression to extract the directory from the last file read and 
#   place it in the variable dir
    regexp {(.*)/[^/]+} $g_filename tmp dir
    catch {destroy .file}
# configure the title of the file selection box based on type
    switch -glob $type {
        b {set t "a Binary"}
        tcl {set t "a Tcl"}
        gcl {set t "a gcl"}
        gmod {set t "a genie module"}
        a? {set t "an ASCII"}
        default {set t "a data"}
    }
    tixExFileSelectDialog .file -title "Select $t file" -command "g_get_filename $type"
# configure the list of file extensions based on type
    switch -glob $type {
        b {.file subwidget fsbox config -filetypes {
            {{*.*}      {All files (*.*)}}
            {{*.raw}    {RAW files (*.raw)}}
            {{*.nxs}    {Nexus files (*.nxs)}}} -pattern {*.raw *.nxs} \
                -dir $g_pd(data_dir)           
        }
        tcl {.file subwidget fsbox config -filetypes {
            {{*.tcl}    {TCL files (*.tcl)}}} -pattern {*.tcl} \
                -dir $g_pd(ext_dir)    
        }
        gcl {.file subwidget fsbox config -filetypes {
            {{*.gcl}    {TCL files (*.gcl)}}} -pattern {*.gcl} \
                -dir $g_pd(gcl_dir)
        }        
        gmod {.file subwidget fsbox config -filetypes {
            {{*.so}      {genie modules (*.so)}}} -pattern {*.so} \
                -dir $g_pd(gmod_dir)    
        } 
        a? {.file subwidget fsbox config -filetypes {
            {{*.*}      {All files (*.*)}}
            {{*.dat}    {ASCII files (*.dat)}}
            {{*.txt}    {ASCII files (*.txt)}}
            {{*.asc}    {ASCII files (*.asc)}}} -pattern {*.dat *.txt *.asc} \
                -dir $g_pd(data_dir)
        }
        default {.file subwidget fsbox config -filetypes {
            {"*.$g_Imodules($type)"     {module data files}}} \
                -pattern "*.$g_Imodules($type)" -dir $g_pd(data_dir)
        }   
    }
# If dir is not the current directory then set the directory to $dir
    if { "$dir" != "." } { 
        .file subwidget fsbox config -directory $dir
    }
# popup the file selection box
    .file popup
}            

proc g_get_filename {type filename} {
# called by the file selection box when the user Ok's their selection
# type is passed through from the previous procedure
# filename contains the full path of the file selected
    global g_filename
# fix filename for NT
    regsub -all {\\} $filename / g_filename
# call the relevant read procedure based on type
    switch -glob $type {
        a? {g_read_ascii $type}
        b {g_read_binary}
        tcl {g_load_extension}
        gcl {g_load_gcl}
        gmod {g_load_module}
        default {g_input_module $type}
    }
}

proc g_load_extension {} {
    global g_filename
    source $g_filename
}

proc g_load_gcl {} {
    global g_filename
    gcl "load \"$g_filename\""
}

proc g_load_module {} {
    global g_filename g_Imodules g_Tmodules g_Mlang g_current
    gcl "module/load \"$g_filename\""
    gcl "wa\[101\]=module:execute:c(\"gmodule_query\")"
    set length [gcl "print length(wa\[101\].symbol)" -out]
# if no symbols then try executing as a FORTRAN module instead!
    if {$length == 0} {
        gcl "wa\[101\]=module:execute(\"gmodule_query\")"
        set length [gcl "print length(wa\[101\].symbol)" -out]
    }    
    for {set count 1} {$count <= $length} {incr count} {
        set symbol [gcl "print wa\[101\].symbol\[$count\]" -out]
        set type [gcl "print wa\[101\].type\[$count\]" -out]        
        set descrip [gcl "print wa\[101\].descrip\[$count\]" -out]
        set language [gcl "print wa\[101\].language\[$count\]" -out]
        set g_Mlang($symbol) $language
        if {$type == "INPUT"} {
            set extent [gcl "print wa\[101\].extent\[$count\]" -out]
            .m.file.f.read add command -label $descrip \
                -command "g_read_data $symbol"
            set g_Imodules($symbol) $extent
        }
        if {$type == "TRANSFORM"} {
            if {$g_current > 0} {
                g_Mactivate .m.operations.oper 3
            }
            set g_Tmodules($descrip) $symbol
        }  
    }
}

proc g_input_module { type } {
    global g_filename
    set get .getData
    g_winPosition $get
    wm title $get "Read Data"
    tixControl $get.spectra -label "Block/Spectrum:" -integer true
    $get.spectra subwidget entry config -textvariable gw_spectra
    tixButtonBox $get.box -orientation horizontal
    $get.box add ok -text Ok -command "g_input_moduleOk $type \$gw_spectra;\
        catch {destroy $get}"
    $get.box add help -text Help
    $get.box add cancel -text Cancel -command "catch {destroy $get}"
    pack $get.spectra $get.box -side top -expand yes -fill x
# find the number of blocks in the chosen file
    set max [gcl "print nblocks(\"$g_filename\")" -out]
    $get.spectra config -value 1 -min 1 -max $max
    bind $get <Return> "$get.box.ok invoke"
}

proc g_module_transform {} {
    global g_current g_set g_Tmodules
    set modules [array names g_Tmodules]
# get the size of the set information array 
    set size [array size g_set]
# divide size by 10 to get the number of used sets
    set size [expr $size/10]
# find the number of the next set
    set next [gcl "print count+1" -out]
    set t .transform
    g_winPosition $t
    frame $t.m
    label $t.m.eq -text "Set $next = "
    tixScrolledListBox $t.m.set1 -scrollbar auto
    set set1 [$t.m.set1 subwidget listbox]
    $set1 config -exportselection false
    for {set c 1} {$c <= $size} {incr c 1} {
        $set1 insert end "Set $c"
    }
    label $t.m.with -text "transformed with:"
    tixScrolledListBox $t.m.symbol
    set symbol [$t.m.symbol subwidget listbox]
    $symbol config -exportselection false
    foreach op $modules {
        if {$op != "none"} {
            $symbol insert end $op
        }
    }
    tixButtonBox $t.box -orientation horizontal
    $t.box add ok -text Ok -command "g_module_transformOk $set1 $symbol;\
        catch {destroy $t}"
    $t.box add help -text Help
    $t.box add cancel -text Cancel -command "catch {destroy $t}"
    pack $t.m.eq $t.m.set1 $t.m.with $t.m.symbol -side left -fill both\
         -expand yes
    pack $t.m $t.box -side top -fill both -expand yes
    bind $t <Return> "$t.box.ok invoke"
    $set1 select set 0
    $symbol select set 0
}

proc g_module_transformOk {set1 symbol} {
    global g_current g_Tmodules g_Mlang
    set set1 [$set1 curselection]
    set set1 [expr $set1 + 1]
    set module [$symbol curselection]
    set module [$symbol get $module]
# create the new set
    gcl "tmp=count+1"
    if {$g_Mlang($g_Tmodules($module)) == "C"} {
        set output [gcl "wa\[tmp\]=module:execute:c(\"$g_Tmodules($module)\",\
            wa\[$set1\])"]
    } else {
        set output [gcl "wa\[tmp\]=module:execute(\"$g_Tmodules($module)\",\
            wa\[$set1\])"]
    }    
    if {[g_testOutput $output]} {
        gcl "count=tmp"
        set g_current [gcl "print count" -out]
# set the workspace
        g_setw $g_current $g_current
# include a comment in the filename slot
        g_setf $g_current "Set $set1 transformed with $module"
# set all other set details to _
        g_setb $g_current _
        g_setTemplate $g_current [g_getTemplate $set1] 
    } 
}

proc g_input_moduleOk { type spectra } {
# actually read data into a set 
    global g_filename g_current g_Mlang
    gcl "count=count+1"
    set g_current [gcl "print count" -out]
    g_setw $g_current $g_current
    g_setf $g_current $g_filename
    g_setTemplate $g_current Default 
    gcl "wa\[101\]=fields()"
    gcl "wa\[101\].file=\"$g_filename\""
    gcl "wa\[101\].block=$spectra"
    if {$g_Mlang($type) == "C"} {
        set output [gcl "wa\[count\]=module:execute:c(\"$type\",wa\[101\])"]
    } else {
        set output [gcl "wa\[count\]=module:execute(\"$type\",wa\[101\])"]
    }    
    g_setb $g_current $spectra
    g_testOutput $output
}

proc g_read_ascii { type } {
# prompts for options and conformation of the ascii read
# type is again passed down from above
    set get .getData
    g_winPosition $get
    wm title $get "Read Ascii Data"
    tixComboBox $get.f -label Fields: -variable gw_f
    $get.f insert end "x,y"
    $get.f insert end "x,y,e"
# include a separator or format option based on the type
    if {$type == "a1"} {
        tixLabelEntry $get.s -label "Separator:"    
        $get.s subwidget entry config -textvariable gw_s
        $get.s subwidget entry delete 0 end
        $get.s subwidget entry insert 0 \$whitespace
    } else {
        tixLabelEntry $get.s -label "Format string:"
        $get.s subwidget entry config -textvariable gw_s
        $get.s subwidget entry delete 0 end
    }
    tixLabelEntry $get.c -label "Comment character:"
    $get.c subwidget entry config -textvariable gw_c
    $get.c subwidget entry delete 0 end 
    $get.c subwidget entry insert 0 #     
    tixLabelEntry $get.l -label "No. of lines:"
    $get.l subwidget entry config -textvariable gw_l
    $get.l subwidget entry delete 0 end
    $get.l subwidget entry insert 0 -1
    tixButtonBox $get.box -orientation horizontal
    $get.box add ok -text Ok -command "g_readasc \$gw_f \$gw_s \$gw_l \$gw_c $type;\
        catch {destroy $get}"
    $get.box add help -text Help -command \
        {g_help "reading.htm#Reading ASCII Data"}
    $get.box add cancel -text Cancel -command "catch {destroy $get}"
    pack $get.f $get.s $get.l -side top -expand yes -fill x
    if {$type == "a1"} {
        pack $get.c -expand yes -fill x
    }
    pack $get.box -side top -expand yes -fill x
    $get.f config -value "x,y" 
    bind $get <Return> "$get.box.ok invoke"
}

proc g_readasc {f s l c type} {
# actually reads ascii data into a set
# f contains the fields to read
# s contains the separator or format string
# l contains the number of lines to read, -1 meaning all lines
    global g_filename g_current
    set output [ gcl "handle=asciifile:open(\"$g_filename\",\"$c\");count=count+1" ]
    g_testOutput $output
# call a different gcl command based on type
    if {$type == "a1"} {
# if the separator is $whitespace then do not put quotes around the string
        if {$s == "\$whitespace"} {
            set output [gcl "wa\[count\]=asciifile:readfree(handle,\"$f\",$s,$l)"]
        } else {
            set output [gcl "wa\[count\]=asciifile:readfree(handle,\"$f\",\"$s\",$l)"]
        }
    } else {
        set output [gcl "wa\[count\]=asciifile:readfixed(handle,\"$f\",\"$s\",$l)"]
    }
    g_testOutput $output
    gcl "asciifile/close(handle)"
# set the current set to be the new set
    set g_current [gcl "print count" -out]
# set the workspace array number
    g_setw $g_current $g_current
# if an error array was read then set the e array to .e, otherwise _
    if { $f == "x,y" } {
        g_setTemplate $g_current "2 Column ASCII"
    } else {
        g_setTemplate $g_current "3 Column ASCII"
    }
# set the filename
    g_setf $g_current $g_filename
# set the block number to _ (not from a block)
    g_setb $g_current _
}
        
proc g_read_dae {} {
    global g_filename
    set inst [ gcl { print _getenv("INST_NAME") } -out ]
    set inst [ string tolower $inst ]
    set g_filename /.:/servers/${inst}_dae
    g_read_binary
}

proc g_read_binary {} {
# prompts for options and conformation of the binary read
    global g_filename
    set get .getData
    g_winPosition $get
    wm title $get "Read Binary Data"
    tixControl $get.spectra -label "Block/Spectrum:" -integer true 
    $get.spectra subwidget entry config -textvariable gw_spectra
    tixControl $get.spectra2 -label To: -integer true 
    $get.spectra2 subwidget entry config -textvariable gw_spectra2
    tixButtonBox $get.box -orientation horizontal
    $get.box add ok -text Ok -command "g_readbin \$gw_spectra \$gw_spectra2;\
        catch {destroy $get}"
    set ftype [gcl "print filetype(\"$g_filename\")" -out] 
    if {$ftype == "raw"} {
        $get.box add focus -text Focus -command "g_focus \$gw_spectra \$gw_spectra2;\
            catch {destroy $get}"
    }    
    $get.box add help -text Help -command \
        {g_help "reading.htm#Reading a Binary File"}
    $get.box add cancel -text Cancel -command "catch {destroy $get}"
    pack $get.spectra $get.spectra2 $get.box -side top -expand yes -fill x
# find the number of blocks in the chosen file
    set max [gcl "print nblocks(\"$g_filename\")" -out]
# use this information to restrict spectra numbers to valid block numbers
    $get.spectra config -value 1 -min 1 -max $max
    $get.spectra2 config -value 1 -min 1 -max $max
    bind $get <Return> "$get.box.ok invoke"
}

proc g_focus {spectra spectra2} {
# focus a range of spectra from an ISIS raw file
    set f .focus
    g_winPosition $f
    tixComboBox $f.units -label "Focus units:" -variable gw_unit
    $f.units insert end "D (D-Spacing (A))"
    $f.units insert end "Q (Momentum Transfer (A^-1))"
    $f.units insert end "T (Time of Flight (us))"
    tixButtonBox $f.box -orientation horizontal
    $f.box add ok -text Ok -command "g_focusOk $spectra $spectra2 \$gw_unit;\
        catch {destroy $f}"
    $f.box add help -text Help -command {g_help reading.htm}
    $f.box add cancel -text Cancel -command "catch {destroy $f}"
    pack $f.units $f.box -side top -fill both -expand yes
    bind $f <Return> "$f.box.ok invoke"
}

proc g_focusOk {spectra spectra2 unit} {
    global g_filename g_current
#    set tmp [$unit curselection]
#    set unit [$unit get $tmp]
# scan unit to determine the unit code used by genie
    scan $unit %s unit
    gcl "count=count+1"
    set g_current [gcl "print count" -out]
# set the current set to be the new set
    g_setw $g_current $g_current
# set the filename
    g_setf $g_current $g_filename
    set ftype [gcl "print filetype(\"$g_filename\")" -out] 
    g_setTemplate $g_current "ISIS RAW File"
    set output [gcl "wa\[count\]=focus:$unit ($spectra:$spectra2,\"$g_filename\")"]
    g_setb $g_current "$spectra To: $spectra2"
    g_testOutput $output
}

proc g_readbin {spectra spectra2} {
# actually read binary data into a set
# spectra and spectra2 contain the first and last block numbers 
    global g_filename g_current
    gcl "count=count+1"
    set g_current [gcl "print count" -out]
# set the current set to be the new set
    g_setw $g_current $g_current
# set the filename
    g_setf $g_current $g_filename
    set ftype [gcl "print filetype(\"$g_filename\")" -out] 
    if {$ftype == "raw" || $ftype == "CRPT"} {
        g_setTemplate $g_current "ISIS RAW File"
    } else {
        g_setTemplate $g_current Default 
    }   
# if more than one block is chosen then read 2d data
    if {$spectra2 > $spectra} {
        set output [gcl "wa\[count\]=get($spectra:$spectra2,\"$g_filename\")"]
        g_setb $g_current "$spectra To: $spectra2"
    } else {
        set output [gcl "wa\[count\]=get($spectra,\"$g_filename\")"]
        g_setb $g_current $spectra
    }
    g_testOutput $output
}

proc g_write_data {type} {
# prompts for options and conformation of file read
# type should contain a string representing the file type:
#   wa = ascii, wn = nexus, wgb = genie intermeditate
    set write .writeData
    g_winPosition $write
    wm title $write "Write set to file"
    tixLabelEntry $write.file -label "Filename:"
    $write.file subwidget entry config -textvariable gw_fname
    $write.file subwidget entry delete 0 end
    tixLabelEntry $write.label -label "Label:"
    $write.label subwidget entry config -textvariable gw_label
    $write.label subwidget entry delete 0 end
    tixLabelEntry $write.comment -label "Comment:"
    $write.comment subwidget entry config -textvariable gw_comment
    $write.comment subwidget entry delete 0 end
    tixLabelEntry $write.cc -label "Comment character:"
    $write.cc subwidget entry config -textvariable gw_cc
    $write.cc subwidget entry delete 0 end 
    $write.cc subwidget entry insert 0 #     
    checkbutton $write.new -text "Create new file" -onvalue yes -variable gw_new
    $write.new select
    tixButtonBox $write.box -orientation horizontal
    $write.box add ok -text Ok -command "g_write \$gw_fname \$gw_label \$gw_comment \
        \$gw_cc \$gw_new $type;catch {destroy $write}"
    $write.box add help -text Help -command {g_help writing.htm}
    $write.box add cancel -text Cancel -command "catch {destroy $write}"
    pack $write.file $write.comment -side top -expand yes -fill x
    if {$type == "wa"} {
        pack $write.cc -side top -expand yes -fill x
    } else {
        pack $write.label $write.new -side top -expand yes -fill x
    }
    pack $write.box -side top -expand yes -fill x
    bind $write <Return> "$write.box.ok invoke"
}

proc g_write {fname label comment cc new type} {
# write the current set to a file (only the selected fields if ascii)
# fname contains the filename
# comment contains an optional comment
# new defines whether a new file should be created
    global g_current g_pd
# add working directory to fname
    set fname $g_pd(working_dir)/$fname
# get the genie workspace array position of the current set
    set w [g_getw $g_current]
# call different genie write commands based on type
    switch $type {
        wa {
            set x [g_getx $g_current]
            set y [g_gety $g_current]
            set e [g_gete $g_current]
            set output [ gcl "handle=asciifile:open(\"$fname\",comment=\"$cc\")" ]
    	    g_testOutput $output
            if {$e == "_"} {
                set output [gcl "asciifile/writefree handle \
                    comment=\"$comment\" gv1=wa\[$w\]$x gv2=wa\[$w\]$y"]
            } else {
                set output [gcl "asciifile/writefree handle \
                    comment=\"$comment\" gv1=wa\[$w\]$x gv2=wa\[$w\]$y \
                        gv3=wa\[$w\]$e"]
            }            
            gcl "asciifile/close handle"
        }
        wn {
            if {$new == "yes"} {
                set output [gcl "put/new/nexus wa\[$w\] label=\"$label\" \
                    comment=\"$comment\" file=\"$fname\""]
            } else {
                set output [gcl "put/nexus wa\[$w\] label=\"$label\" \
                    comment=\"$comment\" file=\"$fname\""]
            }
        }
        wgb {
            if {$new == "yes"} {
                set output [gcl "put/new/g3 wa\[$w\] label=\"$label\" \
                    comment=\"$comment\" file=\"$fname\""]
            } else {
                set output [gcl "put/g3 wa\[$w\] label=\"$label\" \
                    comment=\"$comment\" file=\"$fname\""]
            }
        }
    }
    g_testOutput $output
}

proc g_log_data {w x y e clear} {
    global g_gd g_limits
    set xmin $g_limits(xmin)
    set xmax $g_limits(xmax)
    set ymin $g_limits(ymin)
    set ymax $g_limits(ymax)
    set g_limits(xminlog) 0
    set g_limits(xmaxlog) 0
    set g_limits(yminlog) 0
    set g_limits(ymaxlog) 0
    gcl "wa\[101\] = wa\[$w\]"
    if {$g_gd(xlog) == "xlog"} {
        gcl "wa\[101\]$x = log(wa\[101\]$x)"
        gcl "fix wa\[101\]$x 0.0"
        if {$xmin != 0} {
            set xmin log($xmin)
            set g_limits(xminlog) 1
        }
        if {$xmax != 0} {
            set xmax log($xmax)
            set g_limits(xmaxlog) 1
        }
    }
    if {$g_gd(ylog) == "ylog"} {
        gcl "wa\[101\]$y = log(wa\[101\]$y)"
        gcl "fix wa\[101\]$y 0.0"
        if {$ymax != 0} {
            set ymax log($ymax)
            set g_limits(ymaxlog) 1
        }
        if {$ymin != 0} {
            set ymin log($ymin)
            set g_limits(yminlog) 1
        }
        if {$e != "_"} {
            gcl "wa\[101\]$e = wa\[$w\]$e / wa\[$w\]$y"
            gcl "fix wa\[101\]$e 0.0" 
        }           
    }
    if {$clear == 1} {
        gcl "device/clear"
        gcl "win=win_scaled(0.2,0.8,0.2,0.8,$xmin,$xmax,$ymin,$ymax)" 
    }
}

proc g_plot_data {clear} {
    global g_gd g_current g_set g_limits
# retrieve the necessary set information
    set x [g_getx $g_current]
    set y [g_gety $g_current]
    set e [g_gete $g_current]
    set w [g_getw $g_current]
    set t $g_set($g_current,t)
    set xl $g_set($g_current,xl)
    set yl $g_set($g_current,yl)
# find the length of both x and y arrays
    set xlen [gcl "print length(wa\[$w\]$x)" -out]
    set ylen [gcl "print length(wa\[$w\]$y)" -out]
    if {$g_limits(auto) == 1} {
        set g_limits(xmin) [gcl "print min(wa\[$w\]$x)" -out]
        set g_limits(xmax) [gcl "print max(wa\[$w\]$x)" -out]
        set g_limits(ymin) [gcl "print min(wa\[$w\]$y)" -out]
        set g_limits(ymax) [gcl "print max(wa\[$w\]$y)" -out]
        set g_limits(auto) 0
    } 
    g_log_data $w $x $y $e $clear
    set w 101
    if {$clear == 1} {
        gcl "axes/draw/$g_gd(xlog) /$g_gd(ylog) $g_gd(line_colour) $g_gd(charsize) \
            $g_gd(quality) $g_gd(lineThickness) $g_gd(type)"
        if {$t != "_"} {
            gcl "title/draw wa\[$w\]$t $g_gd(text_colour) $g_gd(charsize) \
                $g_gd(quality)"
        }
        if {$xl != "_"} {
            gcl "labels/draw xlabel=wa\[$w\]$xl ylabel=\" \" \
                colour=$g_gd(text_colour) size=$g_gd(charsize) \
                    font=$g_gd(quality)"
        }
        if {$yl != "_"} {
            gcl "labels/draw xlabel=\" \" ylabel=wa\[$w\]$yl \
                colour=$g_gd(text_colour) size=$g_gd(charsize) \
                    font=$g_gd(quality)"
        }
    }
# if histogram data then use binning
    if {$xlen > $ylen} {
# hack, to avoid using an e array
        gcl "tmp = wa\[$w\]$y"
        gcl "groupbins wa\[$w\]$x wa\[$w\]$y tmp $g_gd(binning)"
# if point mode then convert to point data
        if {$g_gd(mode) == "mode:point"} {
            gcl "centre_bins wa\[$w\]$x"
        }
    }
# if the x array is bigger than the y then plot a histogram
    if {$xlen > $ylen} {
        if {$g_gd(mode) == "mode:point"} {
            set output [gcl "histogram/centre/draw wa\[$w\]$x wa\[$w\]$y \
                $g_gd(plot_colour) $g_gd(type) $g_gd(lineThickness)"]
        } else {
            set output [gcl "histogram/draw wa\[$w\]$x wa\[$w\]$y \
                $g_gd(plot_colour) $g_gd(type) $g_gd(lineThickness)"]
        }
    } else {
# otherwise plot a line graph
        set output [gcl "graph/draw wa\[$w\]$x wa\[$w\]$y $g_gd(plot_colour) \
            $g_gd(type) $g_gd(lineThickness)"]
    }
    g_testOutput $output
# add a graticule if option set
    if {$g_gd(graticule) == 1} {
        gcl "graticule/draw/$g_gd(xlog) /$g_gd(ylog) $g_gd(line_colour) \
            $g_gd(charsize) $g_gd(quality) $g_gd(lineThickness) $g_gd(type)"
    }
}

proc g_select_set {} {
# select a set and make it the current set
    global g_current g_set g_os
# synchronise g_current with GCL in case somebody added a new set from the console
    set g_current [ gcl "print count" -out ]
# get the size of the set information array
    set size [array size g_set]
# divide size by 10 to get the number of used sets
    set size [expr $size/10]
    set sel .selectSet
    g_winPosition $sel
    wm title $sel "Select Set"
# format column heading using c prinf style syntax
    set setid [format "%s     %-60s     %10s" Set Filename/Comment Block]
    tixScrolledListBox $sel.set -scrollbar auto
    set listbox [$sel.set subwidget listbox]
# use a courier font to avoid spacing problems
	if {$g_os == "WINNT" | $g_os == "WIN9X"} {
   		label $sel.lab -text $setid -font boldTitle
    	$listbox config -font listText
	} else {
		label $sel.lab -text $setid -font -*-courier-bold-r-*-*-12*
		$listbox config -font -*-courier-medium-r-*-*-12*
	}
# get details for each set, format them, and insert into listbox
    for {set c 1} {$c <= $size} {incr c 1} {
        set fname [g_getf $c]
        set block [g_getb $c]
        if {$block != "_"} {
            set setid [format "%3i     %-60s     %8s" $c $fname $block]
        } else {
            set setid [format "%3i     %-60s" $c $fname]
        }
        $listbox insert end $setid
    }
    tixButtonBox $sel.box -orientation horizontal
# select and highlight the current set in the listbox
    $listbox select set [expr $g_current - 1]
    $sel.box add ok -text Ok -command "g_changeSet $listbox;catch {destroy $sel}"
    $sel.box add help -text Help -command {g_help selectset.htm}
    $sel.box add cancel -text Cancel -command "catch {destroy $sel}"
    pack $sel.lab $sel.set $sel.box -fill x -expand yes
    bind $sel <Return> "$sel.box.ok invoke"
}

proc g_changeSet {listbox} {
# actually change the current set
# listbox should contain the path of the listbox
    global g_current
# find out which line in the listbox was selected
    set set [$listbox curselection]
# make the current set this value + 1
    set g_current [expr $set + 1]
# update the set status line
    g_setinfo _ "$g_current,block" _
    g_checkState
}    

proc g_picture {} {
# select and choose a previously drawn genie picture
# get the size of the picture list
    set size [smalltalk "(gXUUgraphics at: #gXcurrentgraphics) getPListSize prn!" -out]
    set sel .selectPicture
    g_winPosition $sel
    wm title $sel "Select Picture"
    tixScrolledListBox $sel.pic -scrollbar auto
    set listbox [$sel.pic subwidget listbox]
# create an entry in the listbox for each picture
    for {set c 1} {$c <= $size} {incr c 1} {
        $listbox insert end "Picture $c"
    }
    tixButtonBox $sel.box -orientation horizontal
    $sel.box add ok -text Ok -command "g_pictureOk $listbox;catch {destroy $sel}"
    $sel.box add help -text Help -command {g_help picture.htm}
    $sel.box add cancel -text Cancel -command "catch {destroy $sel}"
    pack $sel.pic $sel.box -fill x
# select the current picture
    $listbox select set [expr $size - 1]
    bind $sel <Return> "$sel.box.ok invoke"
}

proc g_pictureOk {listbox} {
# redraw the chosen picture
# listbox should contain the path of the listbox
# find out which line in the listbox was selected
    set pic [$listbox curselection]
    set pic [expr $pic + 1]
# execute gcl redraw command with pic
    gcl "redraw pic($pic)"
}

proc g_newBrowse {} {
    global g_current g_template g_tkDir g_pd
    set oldTemplate [g_getTemplate $g_current]

    set pwd [pwd]
    set userDir $g_pd(template_dir)
    cd $userDir
    foreach fname [glob -nocomplain *.gtp] {
        set fileid [open $fname r] 
        set tname [gets $fileid]
        close $fileid
        set g_template($tname) $userDir/$fname
    }
    cd $pwd

    set tnames [array names g_template]
    
    set b .browseData
    g_winPosition $b
    wm title $b "Select Data"
    tixComboBox $b.template -label Template -command g_selectTemplate
    foreach tname $tnames {
        $b.template insert end $tname
    }
    frame $b.f
    frame $b.l
    label $b.l.fields -text "Field Label"
    listbox $b.f.fields -exportselection false -height 8
    label $b.l.values -text "Field Location" 
    listbox $b.f.values -exportselection false -height 8
    bind $b.f.values <2> {g_browseValue %W %y %X %Y}
    
    g_selectTemplate $oldTemplate

    bind $b.f.fields <1> "g_copySelection $b.f.fields $b.f.values"
    bind $b.f.fields <B1-Motion> "g_copySelection $b.f.fields $b.f.values"
    bind $b.f.values <1> "g_copySelection $b.f.values $b.f.fields"
    bind $b.f.values <B1-Motion> "g_copySelection $b.f.values $b.f.fields"
    bindtags .browseData.f.fields {Listbox .browseData.f.fields .browseData all}
    bindtags .browseData.f.values {Listbox .browseData.f.values .browseData all}
    tixButtonBox $b.box
    $b.box add ok -text Ok -command "catch {destroy $b}"
    $b.box add help -text Help -command "g_help templates.htm"
    $b.box add modify -text Modify -command "g_modifyTemplate \"$oldTemplate\""
    $b.box add cancel -text Cancel -command "g_setTemplate $g_current \
        \"$oldTemplate\";catch {destroy $b}"
    pack $b.template $b.l $b.f $b.box -fill both -expand yes
    pack $b.l.fields $b.l.values -side left -fill both -expand yes
    pack $b.f.fields $b.f.values -side left -fill both -expand yes
    $b.template config -value $oldTemplate
    bind $b <Return> "$b.box.ok invoke"
}

proc g_selectTemplate {tname} {
    global g_template g_current g_fieldNames
    g_setTemplate $g_current $tname
    set path .browseData
    $path.f.fields delete 0 end
    $path.f.values delete 0 end 
    set fileid [open $g_template($tname) r] 
    set tmp [gets $fileid]
    set data [gets $fileid]
    close $fileid
    set size [llength $data]
    for {set count 0} {$count < $size-1} {incr count} {
        $path.f.fields insert end $g_fieldNames([lindex $data $count])
        set count [incr count]
        $path.f.values insert end [lindex $data $count]
    }
}

proc g_copySelection {box1 box2} {
    set line [$box1 curselection]
    if {$line != ""} {
        $box2 select clear active
        $box2 select set $line
        $box2 activate $line
    }
}    

proc g_modifyTemplate {oldTemplate} {
    global g_current g_fieldNames g_fieldIDs g_tkDir
    set b .browseData
    g_deactivate $b.template
    destroy $b.box
    frame $b.f2
    tixScrolledHList $b.f2.tree -options {
        hlist.separator .
        hlist.drawBranch true
        hlist.showBranch true
        hlist.indent 20
        hlist.wideSelection false
        hlist.height 8
    }
    set tlist [$b.f2.tree subwidget hlist]
    g_createTree Array
    bind $tlist <2> {g_browseValue %W %y %X %Y} 
    frame $b.f3
    button $b.f3.remove -text Remove -command "g_modifyRemove $b"
    label $b.f3.down -bitmap @$g_tkDir/down.xbm
    button $b.f3.add -text Add -command "g_modifyAdd $b"
    label $b.f3.up -bitmap @$g_tkDir/up.xbm
    listbox $b.f2.fields -height 8
    set size [$b.f.fields index end]
    set tfields ""
    for {set count 0} {$count < $size} {incr count} {
        set field [$b.f.fields get $count]
        lappend tfields $g_fieldIDs($field)
    }
    foreach field {x y e t xl yl} {
        if {[lsearch $tfields $field] == -1} {
            $b.f2.fields insert end $g_fieldNames($field)
        }
    }
    checkbutton $b.all -text "Show all fields" -onvalue Object -offvalue Array \
        -variable gw_all -command "g_createTree \$gw_all"
    $b.all deselect
    tixButtonBox $b.box
    $b.box add ok -text Ok -command g_createTemplate
    $b.box add help -text Help -command "g_help templates.htm"
    $b.box add cancel -text Cancel -command "g_setTemplate $g_current \
        \"$oldTemplate\";catch {destroy $b}"      
    pack $b.f3.remove $b.f3.down $b.f3.add $b.f3.up -side left -fill both \
        -expand yes
    pack $b.f2.fields $b.f2.tree -side left -fill both -expand yes
    pack $b.f3 $b.f2 $b.all $b.box -fill both -expand yes
    bind $b <Return> "$b.box.ok invoke"
}

proc g_createTree {type} {
    global g_current
    set w [g_getw $g_current]
    set fields [ smalltalk "(gXwa at: $w) getinstarrays: $type!" -list ]
    set fields [concat . $fields]
    set folder [tix getimage folder]
    set b .browseData
    set tlist [$b.f2.tree subwidget hlist]
    $tlist delete all
    foreach field $fields {
        $tlist add $field -itemtype imagetext -image $folder -text $field
    }
}

proc g_createTemplate {} {
    set c .createTemplate
    g_winPosition $c
    tixLabelEntry $c.tname -label "Template name:"
    $c.tname subwidget entry config -textvariable gw_tname
    $c.tname subwidget entry delete 0 end
    tixLabelEntry $c.fname -label "File name:"
    $c.fname subwidget entry config -textvariable gw_fname
    $c.fname subwidget entry delete 0 end    
    tixButtonBox $c.box
    $c.box add ok -text Ok -command "g_saveTemplate \$gw_tname \$gw_fname;\
        catch {destroy $c}"
    $c.box add help -text Help -command {g_help templates.htm}
    $c.box add cancel -text Cancel -command "catch {destroy $c}"
    pack $c.tname $c.fname $c.box -fill both -expand yes
}  

proc g_saveTemplate {tname fname} {
    global g_pd g_current g_template g_fieldIDs g_directorySeparator
    set path .browseData
    set size [$path.f.fields index end]
    set new ""
    for {set count 0} {$count < $size} {incr count} {
        set field [$path.f.fields get $count]
        set value [$path.f.values get $count]
        lappend new $g_fieldIDs($field)
        lappend new $value
    }    
    append fname .gtp
    set userDir $g_pd(template_dir)
    set fileid [open ${userDir}${g_directorySeparator}${fname} {WRONLY CREAT}]
    puts $fileid $tname
    puts $fileid $new
    close $fileid
    set g_template($tname) ${userDir}${g_directorySeparator}${fname}
    g_setTemplate $g_current $tname
    catch {destroy $path}
}

proc g_modifyAdd {path} {
    set tlist [$path.f2.tree subwidget hlist]   
    set field [$path.f2.fields curselection]
    set value [$tlist info selection]
    if {($field != "") && ($value != "")} {
        set field [$path.f2.fields get $field]
        $path.f.fields insert end $field
        $path.f.values insert end $value
        $path.f2.fields delete active
    }
}

proc g_modifyRemove {path} {
    set field [$path.f.fields curselection]
    if {$field != ""} {
        set value [$path.f.fields get $field]
        $path.f.fields delete $field
        $path.f.values delete $field
        $path.f2.fields insert end $value
    }
}

proc g_browseValue {widget y X Y} {
# displays the value of a set field as a 'tool tip'
# widget is the path name of the hlist widget
# y is the y position of the cursor within the widget
# X and Y are the screen coordinates of the cursor
    global g_current
# find the neatest field to the cursor
    set field [$widget nearest $y]
    if {$widget == ".browseData.f.values"} {
        set field [$widget get $field]
    }
# if it is not the root then continue
    if {$field != "."} {
        set w [g_getw $g_current]
        catch {destroy .value}
        toplevel .value
# move window to the cursor position
        wm geometry .value +$X+$Y
# make sure the window is not too small
        wm minsize .value 150 20
# remove borders and decoration
        wm overrideredirect .value 1
# make window a child of main, so that it is tied to the main aplication
        wm transient .value .
        gcl "tmp=wa\[$w\]$field"
# invoke smalltalk method to print the fields value
        set value [smalltalk "gXtmp printfield!" -out]
# display the value with a yellow label
        label .value.label -text "$field = $value" -bg yellow
# when the mouse leaves the window destroy it
        bind .value <Leave> {destroy .value}
        pack .value.label -fill x
    }
}

proc g_zoom {} {
    global g_limits
# zoom the display
    gcl "__graphics.currentwindow=win"
# call getcursor once to get the first corner
    gcl "c1=getcursor()"
# call again from the previous coordinates to get the second corner
    gcl "c2=getcursor(c1.w_x,c1.w_y)"
    gcl "device/clear"
# create a new picture
    gcl "pic_add item=win"
# call win_alter alter to rezize the window
    gcl "win_alter g_min(c1.w_x,c2.w_x) g_max(c1.w_x,c2.w_x) \
        g_min(c1.w_y,c2.w_y) g_max(c1.w_y,c2.w_y) object=win"
    set g_limits(xmin) [gcl "print c1.w_x" -out]
    set g_limits(xmax) [gcl "print c2.w_x" -out]
    set g_limits(ymin) [gcl "print c1.w_y" -out]
    set g_limits(ymax) [gcl "print c2.w_y" -out]   
}

proc g_arthmetic {} {
# create a new set by carring out arthmetic on two existing sets 
    global g_current g_set
# get the size of the set information array 
    set size [array size g_set]
# divide size by 6 to get the number of used sets
    set size [expr $size/10]
# find the number of the next set
    set next [gcl "print count+1" -out]
    set art .arthmetic
    g_winPosition $art
    frame $art.m
    label $art.m.eq -text "Set $next = "
    tixScrolledListBox $art.m.set1 -scrollbar auto
    set set1 [$art.m.set1 subwidget listbox]
    $set1 config -exportselection false
    for {set c 1} {$c <= $size} {incr c 1} {
        $set1 insert end "Set $c"
    }
    tixScrolledListBox $art.m.op
    set opr [$art.m.op subwidget listbox]
    $opr config -exportselection false
    foreach op {+ - / *} {
        $opr insert end $op
    }
    tixScrolledListBox $art.m.set2 -scrollbar auto
    set set2 [$art.m.set2 subwidget listbox]
    $set2 config -exportselection false
    for {set c 1} {$c <= $size} {incr c 1} {
        $set2 insert end "Set $c"
    } 
    tixButtonBox $art.box -orientation horizontal
    $art.box add ok -text Ok -command "g_operate $set1 $opr $set2;\
        catch {destroy $art}"
    $art.box add help -text Help -command {g_help arithmetic.htm}
    $art.box add cancel -text Cancel -command "catch {destroy $art}"
    pack $art.m.eq $art.m.set1 $art.m.op $art.m.set2 -side left -fill both\
         -expand yes
    pack $art.m $art.box -side top -fill both -expand yes
    bind $art <Return> "$art.box.ok invoke"
    $set1 select set 0
    $set2 select set 0
    $opr select set 0 
}

proc g_operate {set1 opr set2} {
# carry out the operation, create a new set, and make it the current one
# set1, opr, and set2 are all listbox path names
    global g_current
# extract the currently selected set numbers and maths symbol
    set set1 [$set1 curselection]
    set set1 [expr $set1 + 1]
    set set2 [$set2 curselection]
    set set2 [expr $set2 + 1]
    set op [$opr curselection]
    set op [$opr get $op]
# create the new set
    gcl "tmp=count+1"
    set output [gcl "wa\[tmp\]=wa\[$set1\] $op wa\[$set2\]"]
    if {[g_testOutput $output]} {
        gcl "count=tmp"
        set g_current [gcl "print count" -out]
# set the workspace
        g_setw $g_current $g_current
# include a comment in the filename slot
        g_setf $g_current "Set $set1 $op Set $set2"
# set all other set details to _
        g_setb $g_current _
        g_setTemplate $g_current [g_getTemplate $set1] 
    } 
}    

proc g_units {} {
# convert an existing set to different units and place the result in a new set
    global g_current g_set
# get the size of the set array
    set size [array size g_set]
# divide by 6 to get the number of sets
    set size [expr $size/10]
# find the number of the next set
    set next [gcl "print count+1" -out]
    set uts .units
    g_winPosition $uts
    frame $uts.m
    label $uts.m.eq -text "Set $next = "
    tixScrolledListBox $uts.m.set -scrollbar auto
    set set [$uts.m.set subwidget listbox]
    $set config -exportselection false
    for {set c 1} {$c <= $size} {incr c 1} {
        $set insert end "Set $c"
    }
    label $uts.m.con -text "Converted to:"
    tixScrolledListBox $uts.m.units -scrollbar auto
    set unit [$uts.m.units subwidget listbox]
    $unit config -exportselection false -width 40
    $unit insert end "C (Channel numbers)"
    $unit insert end "D (D-Spacing (A))"
    $unit insert end "E (Energy (me V))"
    $unit insert end "LAM (Wavelength (A))"
    $unit insert end "Q (Momentum Transfer (A^-1))"
    $unit insert end "SQ ((Momentum Tansfer)^2 (a^-2))"
    $unit insert end "T (Time of Flight (us))"
    $unit insert end "LA1 (Primary flight path wavelength (A))"
    $unit insert end "W (Energy transfer E1-E2 (me V))"
    $unit insert end "WN (Energy transfer E1-E2 (cm-1))"
    $unit insert end "TAU (To reciprocal time-of-flight (us/m))" 
    tixButtonBox $uts.box -orientation horizontal
    $uts.box add ok -text Ok -command "g_convertUnits $set $unit;\
        catch {destroy $uts}"
    $uts.box add help -text Help -command {g_help units.htm}
    $uts.box add cancel -text Cancel -command "catch {destroy $uts}"
    pack $uts.m.eq $uts.m.set $uts.m.con $uts.m.units -side left -fill both\
         -expand yes
    pack $uts.m $uts.box -side top -fill both -expand yes
    bind $uts <Return> "$uts.box.ok invoke"
    $set select set 0
    $unit select set 0
}

proc g_convertUnits {set unit} {
# carry out the conversion, create a new set, and make it the current one
# set and unit are listbox pathnames
    global g_current
# extract the selected unit and set
    set set [$set curselection]
    set set [expr $set + 1]
    set tmp [$unit curselection]
    set unit [$unit get $tmp]
# scan unit to determine the unit code used by genie
    scan $unit %s unit
# create the new set
    gcl "tmp=count+1"
    set output [gcl "wa\[tmp\]=units:$unit (wa\[$set\])"]
    if {[g_testOutput $output]} {
        gcl "count=tmp"    
        set g_current [gcl "print count" -out]
# set the workspace
        g_setw $g_current $g_current
# add a comment
        g_setf $g_current "Set $set in $unit"
# set all other values to _
        g_setb $g_current _
        g_setTemplate $g_current [g_getTemplate $set]
    }
}

proc g_rebin {} {
# rebin one set against another and store the result in a new set
    global g_current g_set
# get the size of the set array
    set size [array size g_set]
# divide by 6 to get the number of sets
    set size [expr $size/10]
# find the number of the next set
    set next [gcl "print count+1" -out]
    set bin .rebin
    g_winPosition $bin
    frame $bin.m
    label $bin.m.eq -text "Set $next = "
    tixScrolledListBox $bin.m.set1 -scrollbar auto
    set set1 [$bin.m.set1 subwidget listbox]
    $set1 config -exportselection false
    for {set c 1} {$c <= $size} {incr c 1} {
        $set1 insert end "Set $c"
    }
    label $bin.m.rb -text "Rebinned against "
    tixScrolledListBox $bin.m.set2 -scrollbar auto
    set set2 [$bin.m.set2 subwidget listbox]
    $set2 config -exportselection false
    for {set c 1} {$c <= $size} {incr c 1} {
        $set2 insert end "Set $c"
    } 
    tixButtonBox $bin.box -orientation horizontal
    $bin.box add ok -text Ok -command "g_bin $set1 $set2;catch {destroy $bin}"
    $bin.box add help -text Help -command {g_help rebining.htm}
    $bin.box add cancel -text Cancel -command "catch {destroy $bin}"
    pack $bin.m.eq $bin.m.set1 $bin.m.rb $bin.m.set2 -side left -fill both\
         -expand yes
    pack $bin.m $bin.box -side top -fill both -expand yes
    bind $bin <Return> "$bin.box.ok invoke"
    $set1 select set 0
    $set2 select set 0
}

proc g_bin {set1 set2} {
# carry out the rebinning, create a new set, and make it the current one
# set1 and set2 are both listbox path names
    global g_current
# extract the numbers of the currently selected sets
    set set1 [$set1 curselection]
    set set1 [expr $set1 + 1]
    set set2 [$set2 curselection]
    set set2 [expr $set2 + 1]
    set x [g_getx $set2]
    gcl "tmp=count+1"
# rebin set1 against the x array of set2
    set output [gcl "wa\[tmp\]=rebin(wa\[$set1\],wa\[$set2\]$x)"]
    if {[g_testOutput $output]} {
        gcl "count=tmp"
        set g_current [gcl "print count" -out]
# set the workspace
        g_setw $g_current $g_current
# put a comment in the filename slot
        g_setf $g_current "Set $set1 rebinned against Set $set2"
# set all other fields to _
        g_setb $g_current _    
        g_setTemplate $g_current [g_getTemplate $set1]
    }
}

proc g_rebin2 {} {
# rebin a set over a given range and store the result in a new set
    global g_current g_set
# get the size of the set array
    set size [array size g_set]
# divide by 6 to get the number of sets
    set size [expr $size/10]
# get the number of the next set
    set next [gcl "print count+1" -out]
    set bin .rebin
    g_winPosition $bin
    frame $bin.m
    label $bin.m.eq -text "Set $next = "
    tixScrolledListBox $bin.m.set1 -scrollbar auto
    set set1 [$bin.m.set1 subwidget listbox]
    for {set c 1} {$c <= $size} {incr c 1} {
        $set1 insert end "Set $c"
    }
    tixLabelEntry $bin.m.min -label "Rebinned from: "
    $bin.m.min subwidget entry config -textvariable gw_min
    $bin.m.min subwidget entry delete 0 end
    tixLabelEntry $bin.m.max -label "to: "
    $bin.m.max subwidget entry config -textvariable gw_max
    $bin.m.max subwidget entry delete 0 end
    tixButtonBox $bin.box -orientation horizontal
    $bin.box add ok -text Ok -command "g_bin2 $set1 \$gw_min \$gw_max;\
        catch {destroy $bin}"
    $bin.box add help -text Help -command {g_help rebining.htm}
    $bin.box add cancel -text Cancel -command "catch {destroy $bin}"
    pack $bin.m.eq $bin.m.set1 $bin.m.min $bin.m.max -side left -expand yes
    pack $bin.m $bin.box -side top -fill both -expand yes
    bind $bin <Return> "$bin.box.ok invoke"
    $set1 select set 0
}

proc g_bin2 {set1 min max} {
# carry out the rebinning, create a new set, and make it the current one
# set1 is the pathname of the listbox
# min and max contain the rebinning range
    global g_current
# extract the set number of the currently selected set
    set set1 [$set1 curselection]
    set set1 [expr $set1 + 1]
# rebin over the given range
    gcl "tmp=count+1"
    set output [gcl "wa\[tmp\]=rebin(wa\[$set1\],$min,$max)"]
    if {[g_testOutput $output]} {
        gcl "count=tmp"
        set g_current [gcl "print count" -out]
# set the workspace
        g_setw $g_current $g_current
# add a comment in the filename slot
        g_setf $g_current "Set $set1 rebinned from $min to $max"
# set all other fields to _
        g_setb $g_current _
        g_setTemplate $g_current [g_getTemplate $set1]
    }
}

proc g_rebin3 {} {
# rebin a set over boundaries and store the result in a new set
    global g_current g_set
# get the size of the set array
    set size [array size g_set]
# divide by 6 to get the number of sets
    set size [expr $size/10]
# find the next set number
    set next [gcl "print count+1" -out]
    set bin .rebin
    g_winPosition $bin
    frame $bin.m
    label $bin.m.eq -text "Set $next = "
    tixScrolledListBox $bin.m.set1 -scrollbar auto
    set set1 [$bin.m.set1 subwidget listbox]
    for {set c 1} {$c <= $size} {incr c 1} {
        $set1 insert end "Set $c"
    }
    label $bin.m.over -text "Rebinned over: "
    frame $bin.m.b
    tixLabelEntry $bin.m.b.bound1 -labelside top -label "Bound"
    $bin.m.b.bound1 subwidget entry config -textvariable gw_b1
    $bin.m.b.bound1 subwidget entry delete 0 end
    tixLabelEntry $bin.m.b.bound3
    $bin.m.b.bound3 subwidget entry config -textvariable gw_b2
    $bin.m.b.bound3 subwidget entry delete 0 end
    tixLabelEntry $bin.m.b.bound5
    $bin.m.b.bound5 subwidget entry config -textvariable gw_b4
    $bin.m.b.bound5 subwidget entry delete 0 end
    frame $bin.m.s
    tixLabelEntry $bin.m.s.step1 -labelside top -label "Step"
    $bin.m.s.step1 subwidget entry config -textvariable gw_s1
    $bin.m.s.step1 subwidget entry delete 0 end
    tixLabelEntry $bin.m.s.step2
    $bin.m.s.step2 subwidget entry config -textvariable gw_s2
    $bin.m.s.step2 subwidget entry delete 0 end
    tixLabelEntry $bin.m.s.step3
    $bin.m.s.step3 subwidget entry config -textvariable gw_s3
    $bin.m.s.step3 subwidget entry delete 0 end
    frame $bin.m.b2
    tixLabelEntry $bin.m.b2.bound2 -labelside top -label "Bound"
    $bin.m.b2.bound2 subwidget entry config -textvariable gw_b2
    $bin.m.b2.bound2 subwidget entry delete 0 end
    tixLabelEntry $bin.m.b2.bound4
    $bin.m.b2.bound4 subwidget entry config -textvariable gw_b4
    $bin.m.b2.bound4 subwidget entry delete 0 end
    tixLabelEntry $bin.m.b2.bound6
    $bin.m.b2.bound6 subwidget entry config -textvariable gw_b6
    $bin.m.b2.bound6 subwidget entry delete 0 end
    frame $bin.l
    foreach type {log lin} {
        radiobutton $bin.l.$type -text $type -value $type -variable gw_log
        pack $bin.l.$type -side left -expand yes
    }
    tixButtonBox $bin.box -orientation horizontal
    $bin.box add ok -text Ok -command "g_bin3 $set1 \$gw_b1 \$gw_s1 \$gw_b2 \$gw_s2 \
		\$gw_b4 \$gw_s3 \$gw_b6 \$gw_log;catch {destroy $bin}"
    $bin.box add help -text Help -command {g_help rebining.htm}
    $bin.box add cancel -text Cancel -command "catch {destroy $bin}"
    pack $bin.m.b.bound1 $bin.m.b.bound3 $bin.m.b.bound5 -side top -expand yes
    pack $bin.m.b2.bound2 $bin.m.b2.bound4 $bin.m.b2.bound6 -side top -expand yes
    pack $bin.m.s.step1 $bin.m.s.step2 $bin.m.s.step3 -side top -expand yes 
    pack $bin.m.eq $bin.m.set1 $bin.m.over -side left -fill both -expand yes
    pack $bin.m.b $bin.m.s $bin.m.b2 -side left -fill both -expand yes
    pack $bin.m $bin.l -expand yes
    pack $bin.box -side top -fill both -expand yes
    $bin.l.lin select  
    bind $bin <Return> "$bin.box.ok invoke"
    $set1 select set 0
}

proc g_bin3 {set1 b1 s1 b2 s2 b4 s3 b6 log} {
# carry out the rebinning, create a new set, and make it the current one
# set1 is a listbox pathame
# b1 - b6 are the boundaries
# log is set to log for log binning or lin for linear binning
    global g_current
# extract the currently selected set
    set set1 [$set1 curselection]
    set set1 [expr $set1 + 1]
    gcl "tmp=count+1"
# call rebin with different arguments based on how many boundaries are given
    if {[string len $b6] > 0} {
        set output [gcl "wa\[tmp\]=rebin:$log (wa\[$set1\],$b1,$s1,$b2,$s2,$b4,$s3,$b6)"]
    } elseif {[string len $b4] > 0} {
        set output [gcl "wa\[tmp\]=rebin:$log (wa\[$set1\],$b1,$s1,$b2,$s2,$b4)"]
    } else {
        set output [gcl "wa\[tmp\]=rebin:$log (wa\[$set1\],$b1,$s1,$b2)"]
    }
    if {[g_testOutput $output]} {
        gcl "count=tmp"
        set g_current [gcl "print count" -out]
        g_setw $g_current $g_current
# add a comment in the filename field
        g_setf $g_current "Set $set1 rebinned"
# set all other values to _
        g_setb $g_current _
        g_setTemplate $g_current [g_getTemplate $set1]
    }
}

proc g_setx {set x} {
# set the x array field of set to the value of x
    global g_set
    set g_set($set,x) $x
}

proc g_sety {set y} {
# set the y array field of set to the value of y
    global g_set
    set g_set($set,y) $y
}

proc g_sete {set e} {
# set the e array field of set to the value of e
    global g_set
    set g_set($set,e) $e
}

proc g_setw {set w} {
# set the workspace array position of set to the value of w
    global g_set
    set g_set($set,wk) $w
}

proc g_setf {set f} {
# set the filename/comment of set to the value of f
    global g_set
    set g_set($set,file) $f
}

proc g_setb {set b} {
# set the block number of set to be the value of b
    global g_set
    set g_set($set,block) $b
}

proc g_setTemplate {set tname} {
    global g_set g_current g_template g_Tmodules
# activate all menu items and buttons which require data but not necessarily 
#   fields to be defined
    if {$g_current < 2} {
        g_activate .b.box.browse
        g_Mactivate .m.file.f 1
        g_Mactivate .m.select.s 0 1
        g_Mactivate .m.operations.oper 0 1 2
        g_Mactivate .m.draw.d 3
        if {[array size g_Tmodules] > 1} {
            g_Mactivate .m.operations.oper 3
        }
    }
    set g_set($set,template) $tname
    set g_set($set,x) _
    set g_set($set,y) _
    set g_set($set,e) _
    set g_set($set,t) _
    set g_set($set,xl) _
    set g_set($set,yl) _
    set fileid [open $g_template($tname) r] 
    set tmp [gets $fileid]
    set data [gets $fileid]
    close $fileid
    set size [llength $data]
    for {set count 0} {$count < $size-1} {incr count} {
        set field [lindex $data $count]
        set count [incr count]
        set value [lindex $data $count]
        set g_set($set,$field) $value
    }
    if {$g_current == $set} {
        g_checkState
    }
}

proc g_getTemplate {set} {
    global g_set
    return $g_set($set,template)
}

proc g_getx {set} {
# return the x array field value of set
    global g_set
    return $g_set($set,x)
}

proc g_gety {set} {
# return the y array field value of set
    global g_set
    return $g_set($set,y)
}

proc g_gete {set} {
# return the e array field value of set
    global g_set
    return $g_set($set,e)
}

proc g_getw {set} {
# return the workspace array number of set
    global g_set
    return $g_set($set,wk)
}

proc g_getf {set} {
# return the filename/comment of set
    global g_set
    return $g_set($set,file)
}

proc g_getb {set} {
# return the block number of set
    global g_set
    return $g_set($set,block)
}        

proc g_graphics_defaults {} {
# allow the graphics defaults to be displayed and altered
    global g_gd
# turn the defaults array into a list
    set list [array get g_gd]
# call the right-click alter procedure directly with this list
    g_alter noObject $list
    array set g_gd $list
    g_update_gcl_defaults
}

proc g_program_defaults {} {
    global g_pd
    set list [array get g_pd]
    g_alter noObject2 $list
}

proc g_saveDefaults {quiet} {
    global g_gd g_pd g_defaultsFile g_defaultsModify
# convert newline characters in g_pd(scratch) to dummy characters before saving
	set temp [format "%c" 6]
	set scratch $g_pd(scratch)
	regsub -all "\n" $scratch $temp g_pd(scratch)
    set fileid [open $g_defaultsFile {WRONLY CREAT}]
    puts $fileid [array get g_gd]
    puts $fileid [array get g_pd]
    close $fileid
	set g_pd(scratch) $scratch
    if {$quiet != 1} {
        tk_dialog .dialog "Save Defaults..." "Defaults saved in $g_defaultsFile" "" 0 Ok
    }
    set g_defaultsModify 0
}

proc g_hardcopy {} {
    global g_gd g_pd g_directorySeparator
    set hcopy .hardcopy
    g_winPosition $hcopy
    wm title $hcopy "Produce Hardcopy"
    tixLabelEntry $hcopy.file -label "$g_gd(hardcopy) filename: "
    $hcopy.file subwidget entry config -textvariable gw_fname
    $hcopy.file subwidget entry delete 0 end
    $hcopy.file subwidget entry insert end "$g_pd(working_dir)${g_directorySeparator}GENIE.$g_gd(hardcopy)"
    tixButtonBox $hcopy.box -orientation horizontal
    set printers [smalltalk "Smalltalk getPrinters prn!" -list]
    set size [llength $printers]
    label $hcopy.desc -width 60
    tixComboBox $hcopy.prin -label Printer: -variable gw_pname \
        -browsecmd g_choosePrinter
    set pdes "NONE"
    for {set c 0} {$c < $size} {incr c 2} {
	set pname [ lindex $printers $c ]
        $hcopy.prin insert end $pname
	if { [ string match $pname $g_pd(printer) ] } {
	    set c1 [ expr $c + 1 ]
	    set pdes [ lindex $printers $c1 ]
	}
    }
    checkbutton $hcopy.check -text "Print to file" -variable gw_to
    $hcopy.check deselect
    $hcopy.box add ok -text Ok -command "g_hardcopyOk \$gw_fname \$gw_pname \$gw_to;\
        catch {destroy $hcopy}"
    $hcopy.box add help -text Help -command {g_help hardcopy.htm}
    $hcopy.box add cancel -text Cancel -command "catch {destroy $hcopy}"
    pack $hcopy.prin $hcopy.desc $hcopy.check $hcopy.file $hcopy.box \
        -expand yes -fill x
    $hcopy.prin config -value $g_pd(printer)
    $hcopy.desc config -text "Description: $pdes"
    bind $hcopy <Return> "$hcopy.box.ok invoke"
}

proc g_choosePrinter {junk} {
    global g_pd
    set printers [smalltalk "Smalltalk getPrinters prn!" -list]
    set index [.hardcopy.prin subwidget listbox]
    set index [$index curselection]
    set index [expr $index * 2 ]
    set index1 [expr $index + 1 ]   
    .hardcopy.desc config -text "Description: [lindex $printers $index1]"
    set g_pd(printer) [ lindex $printers $index ]
}

proc g_hardcopyOk {fname pname to} {
# create a screen dump of the display area in the file genie.ps
    global g_gd g_pd g_os
    if {$to == 0} {
        set file [gcl {print __tmpfile("genie","ps")} -out]
        regsub -all {\\} $file / file2
        set output [gcl "hardcopy \"$file2\" devtype=\"cps\""]
	set g_pd(printer) $pname
        switch $g_os {
            WINNT {
                exec cmd.exe /c copy $file $pname
            }
            WIN9X {
                exec command.com /c copy $file $pname
            }
            VMS {
                exec PRINT/DELETE/QUEUE=$pname $file
            }
            default {
                exec lpr -P$pname $file
            }
        }
        if {$g_os != "VMS"} {
            set output [gcl "__unlink \"$file2\""]
        }
    } else {
        set output [gcl "hardcopy \"$fname\" \
            devtype=\"$g_gd(hardcopy)\""]
    }
    g_testOutput $output
}

proc g_text {title label command} {
# general purpose procedure to take a text string from the user
# title contains the of the window
# label contains the label of the entry field
# command contains the command to pass the entered text to
    g_winPosition .text
    wm title .text $title
    tixLabelEntry .text.text -label $label
    .text.text subwidget entry config -textvariable gw_text
    .text.text subwidget entry delete 0 end
    tixButtonBox .text.box -orientation horizontal
    .text.box add ok -text Ok -command "$command \$gw_text;catch {destroy .text}"
    .text.box add help -text Help -command {g_help overview.htm}
    .text.box add cancel -text Cancel -command "catch {destroy .text}"
    pack .text.text .text.box -side top -expand yes -fill x
    bind .text <Return> {.text.box.ok invoke}
}

proc g_title {text} {
# displays the title text, above the plot
    global g_gd
    gcl "title/draw \"$text\" $g_gd(text_colour) $g_gd(charsize) $g_gd(quality)"
}

proc g_xlabel {text} {
# adds the xlabel text, to the plot
    global g_gd
    gcl "labels/draw xlabel=\"$text\" ylabel=\" \" colour=$g_gd(text_colour) \
        size=$g_gd(charsize) font=$g_gd(quality)"
}

proc g_ylabel {text} {
# adds the ylabel text, to the plot
    global g_gd
    gcl "labels/draw/yvertical xlabel=\" \" ylabel=\"$text\" colour=\
        $g_gd(text_colour) size=$g_gd(charsize) font=$g_gd(quality)"
}

proc g_draw {clear type} {
# general purpose plot procedure
# clear is the clear display flag
# type is a string representing the type of plot requested:
#   - line, marker, error, or histogram
    global g_gd g_current g_set g_limits
# get the necessary array names
    set x [g_getx $g_current]
    set y [g_gety $g_current]
    set w [g_getw $g_current]
    set e [g_gete $g_current]
    set t $g_set($g_current,t)
    set xl $g_set($g_current,xl)
    set yl $g_set($g_current,yl)    
# find the length of the x and y arrays
    set xlen [gcl "print length(wa\[$w\]$x)" -out]
    set ylen [gcl "print length(wa\[$w\]$y)" -out]
# if clear flag set then clear the display and re-scale the axes
    if {$g_limits(auto) == 1} {
        set g_limits(xmin) [gcl "print min(wa\[$w\]$x)" -out]
        set g_limits(xmax) [gcl "print max(wa\[$w\]$x)" -out]
        set g_limits(ymin) [gcl "print min(wa\[$w\]$y)" -out]
        set g_limits(ymax) [gcl "print max(wa\[$w\]$y)" -out]
        set g_limits(auto) 0
    } 
    g_log_data $w $x $y $e $clear
    set w 101
# centre bins if the data is in histogram format, but a histogram is not wanted
    if {$type != "histogram"} {
        if {($xlen > $ylen) & ($type != "multi")} {
            gcl "centre_bins wa\[$w\]$x"
        }
    } else {
# if histogram data then use binning
        if {$xlen > $ylen} {
            gcl "tmp = wa\[$w\]$y"
            gcl "groupbins wa\[$w\]$x wa\[$w\]$y tmp $g_gd(binning)"
# if point mode then convert to point data
            if {$g_gd(mode) == "mode:point"} {
                gcl "centre_bins wa\[$w\]$x"
            }
        }
    }                  
    set xlen [gcl "print length(wa\[$w\]$x)" -out]
# call different gcl commands to do the plot based on type
    switch $type {
        line {
            set output [gcl "graph/draw  wa\[$w\]$x wa\[$w\]$y \
                $g_gd(plot_colour) $g_gd(type) $g_gd(lineThickness)"]
        }
        marker {
            set output [gcl "markers/draw wa\[$w\]$x wa\[$w\]$y \
                $g_gd(plot_colour) $g_gd(symbol) $g_gd(markersize)"]
        }
        error {
            set output [gcl "errors/draw wa\[$w\]$x wa\[$w\]$y wa\[$w\]$e \
                $g_gd(plot_colour) $g_gd(lineThickness)"]
        }
        histogram {
            if {$xlen == $ylen} {
                set output [gcl "histogram/centre/draw wa\[$w\]$x wa\[$w\]$y \
                    $g_gd(plot_colour) $g_gd(type) $g_gd(lineThickness)"]
            } else {
                set output [gcl "histogram/draw wa\[$w\]$x wa\[$w\]$y \
                    $g_gd(plot_colour) $g_gd(type) $g_gd(lineThickness)"]
            }
        }
        multi {
            if {$g_gd(mode) == "mode:point"} {
                set output [gcl "multi_plot/create/bincentre wa\[$w\]$x \
                    colour=$g_gd(plot_colour) line_type=$g_gd(type) \
                    line_thickness=$g_gd(lineThickness)"]
            } else {
                set output [gcl "multi_plot/create wa\[$w\]$x 2 \
                    colour=$g_gd(plot_colour) line_type=$g_gd(type) \
                    line_thickness=$g_gd(lineThickness)"]
            }
            gcl "multi_plot/spectra wa\[$w\]$y"
#gcl "win_multiplot 0.2 0.8 0.2 0.8"
            gcl "multi_plot/draw" 
        }
    }
    g_testOutput $output
    if {$clear == 1} {        
        gcl "axes/draw/$g_gd(xlog) /$g_gd(ylog) $g_gd(line_colour) $g_gd(charsize) \
            $g_gd(quality) $g_gd(lineThickness) $g_gd(type)"
        if {$t != "_"} {
            gcl "title/draw wa\[$w\]$t $g_gd(text_colour) $g_gd(charsize) \
                $g_gd(quality)"
        }
        if {$xl != "_"} {
            gcl "labels/draw xlabel=wa\[$w\]$xl ylabel=\" \" \
                colour=$g_gd(text_colour) size=$g_gd(charsize) \
                    font=$g_gd(quality)"
        }
        if {$yl != "_"} {
            gcl "labels/draw xlabel=\" \" ylabel=wa\[$w\]$yl \
                colour=$g_gd(text_colour) size=$g_gd(charsize) \
                    font=$g_gd(quality)"
        }
    }
# add a graticule if flag set in defaults
    if {$g_gd(graticule) == 1} {
        gcl "graticule/draw/$g_gd(xlog) /$g_gd(ylog) $g_gd(line_colour) \
            $g_gd(charsize) $g_gd(quality) $g_gd(lineThickness) $g_gd(type)"
    }
}

proc g_contour {} {
# display a contour plot of 2d data
    global g_current g_gd g_set
    set x [g_getx $g_current]
    set y [g_gety $g_current]
    set w [g_getw $g_current]
    set t $g_set($g_current,t)
    gcl "device/clear"
    gcl "win=win_twod(0.2,0.8,0.2,0.8,wa\[$w\]$x,wa\[$w\]$y)"]
    gcl "axes/draw $g_gd(line_colour) $g_gd(charsize) $g_gd(quality) \
        $g_gd(lineThickness) $g_gd(type)"
    gcl "contourObject=contour:draw(wa\[$w\]$y,colour=$g_gd(plot_colour),\
        line_type=$g_gd(type),line_thickness=$g_gd(lineThickness))"
    if {$t != "_"} {
        gcl "title/draw wa\[$w\]$t $g_gd(text_colour) $g_gd(charsize) \
            $g_gd(quality)"
    }
}

proc g_contourArray {} {
# display a contour array plot of 2d data
    global g_current g_gd g_set
    set x [g_getx $g_current]
    set y [g_gety $g_current]
    set e [g_gete $g_current]
    set w [g_getw $g_current]
    set t $g_set($g_current,t)    
    gcl "device/clear"
    gcl "win=win_twod(0.2,0.8,0.2,0.8,wa\[$w\]$x,wa\[$w\]$y)"]
    gcl "axes/draw $g_gd(line_colour) $g_gd(charsize) $g_gd(quality) \
        $g_gd(lineThickness) $g_gd(type)"
    gcl "contourObject=contour_array:draw(wa\[$w\]$e,xarray=wa\[$w\]$x,\
        yarray=wa\[$w\]$y,colour=$g_gd(plot_colour),line_type=$g_gd(type),\
        line_thickness=$g_gd(lineThickness))"
    if {$t != "_"} {
        gcl "title/draw wa\[$w\]$t $g_gd(text_colour) $g_gd(charsize) \
            $g_gd(quality)"
    }
}

proc g_contourLabels {} {
    global g_gd
    gcl "contour_label/draw colour=$g_gd(text_colour) size=$g_gd(charsize) \
        font=$g_gd(quality) object=contourObject"
}

proc g_cell {} {
# display a cell plot of 2d data
    global g_current g_gd g_set
    set x [g_getx $g_current]
    set y [g_gety $g_current]
    set e [g_gete $g_current]
    set w [g_getw $g_current]
    set t $g_set($g_current,t)    
    gcl "table=colourtable:heat(ncolours=75)"
    gcl "device/clear"
    gcl "win=win_twod(0.2,0.8,0.2,0.8,wa\[$w\]$x,wa\[$w\]$y)"]
    gcl "axes/draw $g_gd(line_colour) $g_gd(charsize) $g_gd(quality) \
        $g_gd(lineThickness) $g_gd(type)"
    gcl "cell/draw wa\[$w\]$y table"
    if {$t != "_"} {
        gcl "title/draw wa\[$w\]$t $g_gd(text_colour) $g_gd(charsize) \
            $g_gd(quality)"
    }
}

proc g_cellArray {} {
# display a cell array plot of 2d data
    global g_current g_gd g_set
    set x [g_getx $g_current]
    set y [g_gety $g_current]
    set e [g_gete $g_current]
    set w [g_getw $g_current]
    set t $g_set($g_current,t)    
    gcl "table=colourtable:heat(ncolours=75)"
    gcl "device/clear"
    gcl "win=win_twod(0.2,0.8,0.2,0.8,wa\[$w\]$x,wa\[$w\]$y)"]
    gcl "axes/draw $g_gd(line_colour) $g_gd(charsize) $g_gd(quality) \
        $g_gd(lineThickness) $g_gd(type)"
    set o [gcl "cell_array/draw wa\[$w\]$e table wa\[$w\]$x wa\[$w\]$y"]
    g_testOutput $o
    if {$t != "_"} {
        gcl "title/draw wa\[$w\]$t $g_gd(text_colour) $g_gd(charsize) \
            $g_gd(quality)"
    }
}

proc g_3dPlot {} {
    global g_current g_gd g_set
    set x [g_getx $g_current]
    set y [g_gety $g_current]
    set e [g_gete $g_current]
    set w [g_getw $g_current]
    set t $g_set($g_current,t)    
    gcl "table=colourtable:heat(ncolours=75,nbands=75,diffuse=0.5,shine=0.0,\
        polish=1.0)"
    gcl "eye=dimensions(3)"
    gcl "eye\[1\]=0.5;eye\[2\]=0.5;eye\[3\]=10.0"
    gcl "light=dimensions(3)"
    gcl "light\[1\]=-1.0;light\[2\]=-1.0;light\[3\]=-1.0"
    gcl "lattice=dimensions(3,3)"
    gcl "lattice\[1,1\]=0.2;lattice\[1,2\]=0.2;lattice\[1,3\]=-5.0"
    gcl "lattice\[2,1\]=1.0;lattice\[2,2\]=0.2;lattice\[2,3\]=-5.0"    
    gcl "lattice\[3,1\]=0.2;lattice\[3,2\]=1.0;lattice\[3,3\]=-12.0"    
    gcl "device/clear"
    gcl "win=win_threed(0.2,0.8,0.2,0.8,0.0,1.0,0.0,1.0,\$black,eye,light)" 
    gcl "wa\[101\]=fields()"
    gcl "wa\[101\]$y=wa\[$w\]$y/max(wa\[$w\]$y)"
    set o [gcl "surface/draw lattice wa\[101\]$y 0.0 1.0 0.5 table"]
    g_testOutput $o
    if {$t != "_"} {
        gcl "title/draw wa\[$w\]$t $g_gd(text_colour) $g_gd(charsize) \
            $g_gd(quality)"
    }
    gcl "redraw"
}

proc g_peakfit_poly {} {
    set f .fit
    g_winPosition $f
    wm title $f "Polynomial Peakfit"
    tixLabelEntry $f.deg -label "Degree:"
    $f.deg subwidget entry config -textvariable gw_deg
    $f.deg subwidget entry delete 0 end  
    tixButtonBox $f.box -orientation horizontal
    $f.box add ok -text Ok -command "catch {destroy $f}; g_peakfit poly"
    $f.box add help -text Help -command {g_help peakfit.htm}
    $f.box add cancel -text Cancel -command "catch {destroy $f}"
    pack $f.deg $f.box -fill x
    bind $f <Return> "$f.box.ok invoke"
}

proc g_peakfit {type} {
    global g_current g_gd g_set g_limits g_limits gw_deg
    set x [g_getx $g_current]
    set y [g_gety $g_current]
    set e [g_gete $g_current]
    set w [g_getw $g_current]
    set char A
    .set config -text "Press left mouse button to select peaks, and any other \
        key to finish"
    while {$char == "A"} {
        gcl "c1=getcursor(0.0,0.0)"
        set char [gcl "print c1.char" -out]
        if {$char == "A"} {
            gcl "c2=getcursor(c1.w_x,c1.w_y)"
            set char [gcl "print c2.char" -out]
            if {$char == "A"} {
                set xmin [gcl "print c1.w_x" -out]
                set xmax [gcl "print c2.w_x" -out]
                gcl "win_alter c1.w_x c2.w_x object=win"                
                set g_limits(xmin) $xmin
                set g_limits(xmax) $xmax
                set g_limits(auto) 1
            }
        }
    }    
    set xlen [gcl "print length(wa\[$w\]$x)" -out]
    set ylen [gcl "print length(wa\[$w\]$y)" -out]
    gcl "wa\[101\] = fields()"
    if {$xlen > $ylen} {
        gcl "wa\[101\].x = centre_bins(wa\[$w\]$x)"
    } else {
        gcl "wa\[101\].x = wa\[$w\]$x"
    }
    set imin [gcl "print bracket(wa\[101\].x,$xmin)" -out]
    set imax [gcl "print bracket(wa\[101\].x,$xmax)" -out]
    if {$imin == -1} {
        set imin 1
    }
    if {$imax == -1} {
        set imax [gcl "print length(wa\[101\].x)" -out]
    }
    gcl "wa\[101\].x = wa\[101\].x\[($imin):($imax)\]"
    gcl "wa\[101\].y = wa\[$w\]$y\[($imin):($imax)\]"
    gcl "wa\[101\].e = wa\[$w\]$e\[($imin):($imax)\]"        
    if {$type == "poly"} {
        gcl "g_tmp = dimensions($gw_deg+1)"
        gcl "fill g_tmp 0.0"
        gcl "g_res = peakfit:$type (wa\[101\].x,wa\[101\].y,wa\[101\].e,g_tmp)"        
    } else {
        gcl "g_res = peakfit:$type (wa\[101\].x,wa\[101\].y,wa\[101\].e)"
    }
    gcl "wa\[101\].y = peakgen:$type (wa\[101\].x,g_res.pars)"
    switch $type {
        gauss {
            gcl {g_res.type = "Gaussian"}
            gcl {g_res.desc = "Y = P1x + P2 + P3*Gauss(P4,P5)"}
        }
        gexp {
            gcl {g_res.type = "Gaussian convolved with an exponential"}
            gcl {g_res.desc = "Y = P1x + P2 + P3*Gsexp(P4,P5,P6)"}
        }        
        loren {
            gcl {g_res.type = "Lorentzian"}
            gcl {g_res.desc = "Y = P1x + P2 + P3*Lorentz(P4,P5)"}
        }
        lexp {
            gcl {g_res.type = "Lorentxian convolved with an exponential"}
            gcl {g_res.desc = "Y = P1x + P2 + P3*Lzexp(P4,P5,P6)"}
        }
        voigt {
            gcl {g_res.type = "Voigt"}
            gcl {g_res.desc = "Y = P1x + P2 + P3*Voigt(P4,P5,P6)"}
        }
        vexp {
            gcl {g_res.type = "Voigt convolved with an exponential"}
            gcl {g_res.desc = "Y = P1x + P2 + P3*Vtexp(P4,P5,P6,P7)"}
        }
        poly {
            gcl "g_res.type = \"Polynomial (degree $gw_deg)\""
            gcl {g_res.desc = "Y = P1 + P2*x + P3*x^2 + ..."}
        }                                
    }
    if {$g_gd(plot_colour) == "\$Green"} {
        set colour \$red
    } else {
        set colour \$green
    }
    if {$g_gd(plot_colour) == "\$Blue"} {
        set colour2 \$red
    } else {
        set colour2 \$blue
    }    
    gcl "graph/draw wa\[101\].x wa\[101\].y $colour \$full 2.0"
    if {$type != "poly"} {
        gcl "g_x1 = min(wa\[101\].x)"
        gcl "g_y1 = g_res.pars\[1\]*g_x1 + g_res.pars\[2\]"
        gcl "g_x2 = max(wa\[101\].x)"
        gcl "g_y2 = g_res.pars\[1\]*g_x2 + g_res.pars\[2\]"
        gcl "line/draw g_x1 g_y1 g_x2 g_y2 $colour2 \$dash"
    }
    g_setinfo _ $g_current,block _
    g_showOutput [gcl "__showfit g_res" -out] "Peakfit Results"
}

proc g_showOutput {text title} {
    set o .output
    g_winPosition $o
    wm title $o $title
    text $o.main -height 10 -width 50
    button $o.button -text Ok -command "catch {destroy $o}"
    pack $o.main -fill both -expand true
    pack $o.button
    $o.main insert end $text
    bind $o <Return> "$o.button invoke"
}

proc g_funcplot {} {
    set f .funcplot
    g_winPosition $f
    wm title $f "Function Plot"
    tixLabelEntry $f.func -label "Function (e.g. x*x):"
    $f.func subwidget entry config -textvariable gw_func
    $f.func subwidget entry delete 0 end
    tixLabelEntry $f.xmin -label XMIN:
    $f.xmin subwidget entry config -textvariable gw_xmin
    $f.xmin subwidget entry delete 0 end
    tixLabelEntry $f.xmax -label XMAX:
    $f.xmax subwidget entry config -textvariable gw_xmax
    $f.xmax subwidget entry delete 0 end        
    tixButtonBox $f.box -orientation horizontal
    $f.box add ok -text Ok -command "g_funcplotOk; catch {destroy $f}"
    $f.box add help -text Help -command {g_help function.htm}
    $f.box add cancel -text Cancel -command "catch {destroy $f}"
    pack $f.func $f.xmin $f.xmax $f.box -fill x
    bind $f <Return> "$f.box.ok invoke"
    focus [$f.func subwidget entry]
}

proc g_funcplotOk {} {
    global g_gd g_limits gw_func gw_xmin gw_xmax gw_clear
    gcl "g_funcplot $gw_xmin $gw_xmax \"$gw_func\""
    if {$gw_clear == 1} {
        gcl "device/clear"
        set g_limits(xmin) [gcl "print min(wa\[101\].x)" -out]
        set g_limits(xmax) [gcl "print max(wa\[101\].x)" -out]
        set g_limits(ymin) [gcl "print min(wa\[101\].y)" -out]
        set g_limits(ymax) [gcl "print max(wa\[101\].y)" -out]
        set g_limits(auto) 1    
        gcl "win=win_autoscaled(0.2,0.8,0.2,0.8,wa\[101\].x,wa\[101\].y)"
    }
    gcl "graph/draw wa\[101\].x wa\[101\].y $g_gd(plot_colour) $g_gd(type) \
        $g_gd(lineThickness)"
    gcl "axes/draw $g_gd(line_colour) $g_gd(charsize) $g_gd(quality) \
        $g_gd(lineThickness) $g_gd(type)"
    gcl "title/draw \"f(x)=$gw_func\" $g_gd(text_colour) $g_gd(charsize) \
        $g_gd(quality)"
    gcl "labels/draw xlabel=\"x\" ylabel=\"y\" colour=$g_gd(text_colour) \
        size=$g_gd(charsize) font=$g_gd(quality)"
}

proc g_setinfo {name1 name2 op} {
# called by the g_set trace and g_select_set to update the status line
# name1 is the name of the array - passed by trace (not used)
# name2 is the name of the array element - passed by trace
# op is the operation - passed by trace (not used)
    global g_current
# check to see if the array element contained block, eg. 3,block
# if so then the required information has been set as block always gets set last
     if {$name2 == "$g_current,block"} {
# find the block and filename
        set fname [g_getf $g_current]
        set block [g_getb $g_current]
# if the block is _ then do not show it
        if { $block != "_" } {
# update the status line, .set is the name of the set label widget
            .set config -text "Set: $g_current From: $fname Block/Spectrum: $block"
        } else {
            .set config -text "Set: $g_current From: $fname"
        }
    }
}

proc g_checkState {} {
# checks the state of the current set and activates/deactivates buttons and 
#   menus accordingly.  This procedure is called when the current set changes
    global g_current g_limits
# reset limits
    set g_limits(auto) 1
    # get the values of x, y, and e
    set x [g_getx $g_current]
    set y [g_gety $g_current]
    set e [g_gete $g_current]
# if x or y are not defined then deactivate options which require both x and y
    if { ($x == "_") || ($y == "_") } {
        g_deactivate .b.box.plot .b.box.limits
        g_Mdeactivate .m.file.f.write 0
        g_Mdeactivate .m.operations.oper.rebin 0
        g_Mdeactivate .m.draw.d 0 1
        g_Mdeactivate .m.select.s 2
# else activate them
    } else {
        g_activate .b.box.plot .b.box.limits
        g_Mactivate .m.file.f.write 0
        g_Mactivate .m.operations.oper.rebin 0
        g_Mactivate .m.draw.d 0 1
        g_Mactivate .m.select.s 2
# if e is not defined then deactivate the errors plot option
        if { $e == "_" } {
            g_Mdeactivate .m.draw.d.plot 2
        } else {
            g_Mactivate .m.draw.d.plot 2
        }
    }
}

proc g_annotate {x y} {
# allows the user to interactively annotate the plot, bound to mouse button 2
# x and y are the canvas coordinates of the mouse cursor
# if the alter window exists then annotate is not wanted so do nothing
    if {[winfo exists .alter] != 1} {
        g_winPosition .text
        wm title .text "Annotation text"
# convert to genie coordinates
        g_genieCoordinates
        tixLabelEntry .text.text -label Text:
        .text.text subwidget entry config -textvariable gw_text
        .text.text subwidget entry delete 0 end
        .text.text subwidget entry insert end ($x,$y)
        checkbutton .text.check -text "centre text" -variable gw_cent  
        .text.check select
        tixButtonBox .text.box -orientation horizontal
        .text.box add ok -text Ok -command "g_annotateOk $x $y \$gw_text \$gw_cent;\
            catch {destroy .text}"
        .text.box add help -text Help -command \
            {g_help "overview.htm#Plot Area Interactions"}
        .text.box add cancel -text Cancel -command "catch {destroy .text}"
        pack .text.text .text.check .text.box -side top -expand yes -fill x
        bind .text <Return> {.text.box.ok invoke}
    }
}

proc g_annotateOk {x y text cent} {
# called if the user Ok's the annotation
# x, and y are passed down from above
# text is the text string provided by the user
    global g_gd
    if {$cent == 1} {
        set output [gcl "text/draw/centre $x $y \"$text\" $g_gd(text_colour) \
            $g_gd(charsize) font=$g_gd(quality)"]
    } else {
        set output [gcl "text/draw $x $y \"$text\" $g_gd(text_colour) \
            $g_gd(charsize) font=$g_gd(quality)"]
    }
    g_testOutput $output
}

proc g_limits {} {
    global g_limits g_current
    set l .limits
    g_winPosition $l
    wm title $l "Change Limits"
    if {$g_limits(xmin) == "_"} {
        set x [g_getx $g_current]
        set y [g_gety $g_current]
        set g_limits(xmin) [gcl "print min(wa\[$g_current\]$x)" -out]
        set g_limits(xmax) [gcl "print max(wa\[$g_current\]$x)" -out]
        set g_limits(ymin) [gcl "print min(wa\[$g_current\]$y)" -out]
        set g_limits(ymax) [gcl "print max(wa\[$g_current\]$y)" -out]
    }
    button $l.auto -text "Auto Limits" -command g_limitsAuto
    foreach limit {xmin xmax ymin ymax} {
        tixLabelEntry $l.$limit -label "$limit : "
        $l.$limit subwidget entry config -textvariable gw_$limit
        $l.$limit subwidget entry delete 0 end
        $l.$limit subwidget entry insert end $g_limits($limit)
    }
    tixButtonBox $l.box -orientation horizontal
    $l.box add ok -text Ok -command "g_limitsOk \$gw_xmin \$gw_xmax \$gw_ymin \$gw_ymax;\
        catch {destroy $l}"
    $l.box add help -text Help -command {g_help limits.htm}
    $l.box add cancel -text Cancel -command "catch {destroy $l}"
    pack $l.auto $l.xmin $l.xmax $l.ymin $l.ymax $l.box -expand yes -fill x
    bind $l <Return> "$l.box.ok invoke"
}

proc g_limitsAuto {} {
    global g_current
    upvar gw_xmin gw_xmin
    upvar gw_xmax gw_xmax
    upvar gw_ymin gw_ymin
    upvar gw_ymax gw_ymax
    set x [g_getx $g_current]
    set y [g_gety $g_current]
    set gw_xmin [gcl "print min(wa\[$g_current\]$x)" -out]
    set gw_xmax [gcl "print max(wa\[$g_current\]$x)" -out]
    set gw_ymin [gcl "print min(wa\[$g_current\]$y)" -out]
    set gw_ymax [gcl "print max(wa\[$g_current\]$y)" -out]    
}

proc g_limitsOk {xmin xmax ymin ymax} {
    global g_limits
    set g_limits(xmin) $xmin
    set g_limits(xmax) $xmax
    set g_limits(ymin) $ymin
    set g_limits(ymax) $ymax
}

proc g_genieCoordinates {} {
    global g_limits
    upvar x x
    upvar y y
# find the width and height of the tk canvas
    set xMax [winfo width .p1]
    set yMax [winfo height .p1]
# get limits
    if {$g_limits(xminlog) == 1} {
        set lxMin [expr log10($g_limits(xmin))]
    } else {
        set lxMin $g_limits(xmin)
    }
    if {$g_limits(xmaxlog) == 1} {
        set lxMax [expr log10($g_limits(xmax))]
    } else {
        set lxMax $g_limits(xmax)
    }
    if {$g_limits(yminlog) == 1} {
        set lyMin [expr log10($g_limits(ymin))]
    } else {    
        set lyMin $g_limits(ymin)
    }
    if {$g_limits(ymaxlog) == 1} {
        set lyMax [expr log10($g_limits(ymax))]
    } else {    
        set lyMax $g_limits(ymax)
    }
    set dxMin $g_limits(dxmin)
    set dxMax $g_limits(dxmax)
    set dyMin $g_limits(dymin)
    set dyMax $g_limits(dymax)    
# change y to fit with the genie coordinate system
    set y [expr $yMax - $y]
# adjust values to allow for the border
    set x [expr $x - ($xMax * $dxMin)]
    set y [expr $y - ($yMax * $dyMin)]
    set xMax [expr $xMax * ($dxMax - $dxMin)]
    set yMax [expr $yMax * ($dyMax - $dyMin)]
# finally convert the adjusted canvas coordinates to genie coordinates
    set x [expr ((($lxMax - $lxMin) / $xMax) * $x) + $lxMin]
    set y [expr ((($lyMax - $lyMin) / $yMax) * $y) + $lyMin]
}    

proc g_coordinates {x y} {
    set res [ catch { g_genieCoordinates } ]
    if { $res == 0 } {
        .b.coord.x config -text "x: $x"
        .b.coord.y config -text "y: $y"
    }
}

proc g_setTrace {name1 name2 op} {
    global g_current g_current_set
    set g_current_set wa\[$g_current\]
}                

proc g_getSet {args} {
    global g_set g_current
    set p1 [lindex $args 0]
    if {[llength $args] == 1} {
        if [catch {set w $g_set($p1,wk)}] {
            set w $g_set($g_current,wk)
            set f $g_set($g_current,$p1)
            return wa\[$w\]$f    
        } else {
            return wa\[$w\]
        }
    } else {
        set w $g_set($p1,wk)
        set p2 [lindex $args 1]
        set f $g_set($p1,$p2)
        return wa\[$w\]$f
    }
}

proc g_setCount {} {
    global g_set
    set size [array size g_set]
    return [expr $size/10]
}

proc g_nextSet {} {
    return [g_nextW]
}

proc g_nextW {} {
    return [gcl "print count+1" -out]
}

proc g_createSet {} {
    set s [gcl "count = count + 1;print count" -out]
# make sure g_set is complete
    g_setw $s $s
    g_setTemplate $s Default
    g_setb $s _
    g_setf $s _
    gcl "[g_getSet $s] = fields()"
    return $s
}

proc g_copySet {{set 0}} {
    global g_current
    if {$set == 0} {
        set set $g_current
    }
    set s [gcl "count = count + 1;print count" -out]
    g_setw $s $s
    gcl "[g_getSet $s] = [g_getSet $set]"
    g_setTemplate $s [g_getTemplate $set]
    g_setb $s _
    g_setf $s _    
    return $s
}

proc g_registerSet {set} {
    global g_current
# update g_current
    set g_current $set
# refresh status line
    g_setinfo _ "$g_current,block" _
# check the state
    g_checkState
}

proc g_setLimits {args} {
    global g_limits
    if {[llength $args] == 0} {
        gcl "tmp = __graphics.currentwindow"
        set g_limits(xmin) [smalltalk "gXtmp lxMin prn!" -out]
        set g_limits(xmax) [smalltalk "gXtmp lxMax prn!" -out]
        set g_limits(ymin) [smalltalk "gXtmp lyMin prn!" -out]
        set g_limits(ymax) [smalltalk "gXtmp lyMax prn!" -out]
        set g_limits(dxmin) [smalltalk "gXtmp xMin prn!" -out]
        set g_limits(dxmax) [smalltalk "gXtmp xMax prn!" -out]
        set g_limits(dymin) [smalltalk "gXtmp yMin prn!" -out]
        set g_limits(dymax) [smalltalk "gXtmp yMax prn!" -out]              
    } else {
        set g_limits(xmin) [lindex $args 0]
        set g_limits(xmax) [lindex $args 1]
        set g_limits(ymin) [lindex $args 2]
        set g_limits(ymax) [lindex $args 3]
        set g_limits(dxmin) [lindex $args 4]
        set g_limits(dxmax) [lindex $args 5]
        set g_limits(dymin) [lindex $args 6]
        set g_limits(dymax) [lindex $args 7]        
    }
    set g_limits(auto) 1
}

proc g_exit {} {
    global g_defaultsModify g_pd g_gd g_defaultsFile
    if {[tk_dialog .dialog "Conformation" "Are you sure?" "" 1 Yes No] != 0} {
        return
    }
    if {$g_defaultsModify == 1} {
        if {[tk_dialog .dialog2 "Defaults" "Save Defaults?" "" 1 Yes No] != 0} {
            set name $g_pd(name)
            set email $g_pd(email)
            set fileid [open $g_defaultsFile r]
            array set g_gd [gets $fileid]
            array set g_pd [gets $fileid]
            close $fileid
            set g_pd(name) $name
            set g_pd(email) $email
        }
    }
    set g_pd(devwidth) [winfo width .p1]
    set g_pd(devheight) [winfo height .p1]
	if {[winfo exists .console]} {
		g_saveScratch
	}
    g_saveDefaults 1
    exit    
}

proc g_console {} {
	global g_pd g_new_search
    set c .console
    set g_new_search 1
    catch {destroy $c}
    toplevel $c
    set n ""
    regexp {#.*} [tk appname] n
    wm title $c "Open Genie Scratch Pad $n"
    frame $c.bottom
    text $c.scratch -height 10
	$c.scratch insert 1.0 $g_pd(scratch)
    tixButtonBox $c.buttons
    $c.buttons add run -text Run -command g_runScratch
	$c.buttons add run2 -text "Run Selected" -command g_runSelect
	$c.buttons add help -text Help -command {g_help console.htm}
    $c.buttons add clear -text Clear -command "$c.scratch delete 1.0 end"
    $c.buttons add close -text Close -command "catch {destroy $c}"
    tixLabelEntry $c.bottom.entry -label >>
    $c.bottom.entry subwidget label config -bg white
    set entry [$c.bottom.entry subwidget entry]
    $entry config -borderwidth 0
    tixScrolledText $c.stext -scrollbar y
    set text [$c.stext subwidget text]
    $text config -height 10 -borderwidth 5 -relief ridge
    $text tag configure blue -foreground blue
    $text tag configure red -foreground red
    $text tag configure pink -foreground purple
    button $c.bottom.search -text Search -command g_consoleSearch
    button $c.bottom.help -text Help -command g_consoleHelp
    tixBalloon $c.bal
    $c.bal bind $c.buttons.run -msg "Run all scratch pad code"
    $c.bal bind $c.buttons.run2 -msg "Run any selected scratch pad code"
    $c.bal bind $c.buttons.help -msg "General command console help"
    $c.bal bind $c.buttons.clear -msg "Clear the scratch pad"
    $c.bal bind $c.buttons.close -msg "Close the console"
    $c.bal bind $c.bottom.search -msg "Search command history for >>"    
    $c.bal bind $c.bottom.help -msg "Get help on >> and consult the \
        reference manual"               
    pack $c.scratch -fill both -expand 1
    pack $c.buttons -fill x -expand 0
    pack $c.stext -fill both -expand 1
    pack $c.bottom -fill x -expand 0
    pack $c.bottom.entry -side left -fill x -expand 1
    pack $c.bottom.search $c.bottom.help -side left -expand 0
    bind $entry <KeyPress> {set g_new_search 1}    
    bind $entry <Return> g_consoleEnter
    bind $entry <KeyPress-Up> g_consoleUp
    bind $entry <KeyPress-Down> g_consoleDown
    bind $entry <Control-h> g_commandHelp
    bind $entry <Control-H> g_commandHelp
    bind $entry <KeyPress-F1> g_commandHelp
    bind $entry <Control-k> g_commandSearch
    bind $entry <Control-K> g_commandSearch  
	wm protocol $c WM_DELETE_WINDOW g_saveScratch
}

proc g_consoleEnter {} {
    global g_new_search
    set g_new_search 1
    set c .console
    set entry [$c.bottom.entry subwidget entry]
    set value [$entry get]
    $entry delete 0 end
    set text [$c.stext subwidget text]
    set pos [$text index insert]
    scan $pos "%d.%d" tmp pos
    if {$pos != 0} {
        $text insert end "\n"
    }
    $text insert end $value
    $text insert end "\n"
    g_consoleExec $value 1
}

proc g_consoleUp {} {
    global g_new_search
    set g_new_search 1
    set entry [.console.bottom.entry subwidget entry]
    $entry delete 0 end
    $entry insert 0 [g_previousCommand]    
}

proc g_consoleDown {} {
    global g_new_search
    set g_new_search 1
    set entry [.console.bottom.entry subwidget entry]
    $entry delete 0 end
    $entry insert 0 [g_nextCommand]  
}

proc g_consoleSearch {} {
    global g_new_search g_search_value
    set c .console
    set entry [$c.bottom.entry subwidget entry]    
    if {$g_new_search == 1} {
        set g_search_value [$entry get]
        set result [g_searchHistory $g_search_value new]
        set g_new_search 0
    } else {    
        set result [g_searchHistory $g_search_value old]
    }
    if {$result != ""} {
        $entry delete 0 end
        $entry insert 0 $result
    }  
}

proc g_consoleHelp {} {
    global g_docDir
    set c .console
    set entry [$c.bottom.entry subwidget entry]
    set value [$entry get]
# take part of line up to first space, / or :
    regexp {([^=/:(]+)=?[ ]*([^/ :(]*)} $value tmp val1 val2
    if { $val2 != "" } {
	set value [string tolower $val2]
    } else {
	set value [string tolower $val1 ]
    }
    if {[file exists $g_docDir/Manual/$value.htm]} {
        g_help $value.htm $g_docDir/Manual
    } else {
        g_help index.htm $g_docDir/Manual
    }      
}

proc g_commandHelp {} {
    global g_commands
    array set help_array $g_commands
    set c .console
    set entry [$c.bottom.entry subwidget entry]
    set text [$c.stext subwidget text]
    set value [$entry get]
# take part of line up to first space, / or :
    regexp {([^=/:(]+)=?[ ]*([^/ :(]*)} $value tmp val1 val2
    if { $val2 != "" } {
	set value [string tolower $val2]
    } else {
	set value [string tolower $val1 ]
    }
    if {$value == ""} {
        set value *
    }    
    foreach command [array names help_array $value*] {
        $text insert end $command blue
        $text insert end " - $help_array($command)\n"
    }
    $text yview -pickplace end
}

proc g_commandSearch {} {
    global g_commands
    array set help_array $g_commands
    set c .console
    set entry [$c.bottom.entry subwidget entry]
    set text [$c.stext subwidget text]    
    set value [$entry get]  
    foreach command [array names help_array] {
        if {[string match *$value* $help_array($command)]} {
            $text insert end $command blue
            $text insert end " - $help_array($command)\n"
        }
    }
    $text yview -pickplace end
}

proc g_runScratch {} {
    set c .console
    g_consoleExec [$c.scratch get 1.0 end] 0
}

proc g_saveScratch {} {
	global g_pd
	set c .console
	set g_pd(scratch) [$c.scratch get 1.0 end]
	destroy $c
}

proc g_runSelect {} {
    set c .console
	set range [$c.scratch tag nextrange sel 1.0]
	set i1 [lindex $range 0]
	set i2 [lindex $range 1]
	if {($i1 != "") && ($i2 != "")} {
		g_consoleExec [$c.scratch get $i1 $i2] 0
	}
}

proc g_consoleExec {value history} {
    set c .console
    set text [$c.stext subwidget text]
    g_changeCursor watch .
    if {$history == 1} {
        catch {gcl $value -history} output
    } else {
        catch {gcl $value} output
    }
    g_resetCursor .
# Remove ANSI escape sequences as they don't show up in the widget
    regsub -all {\x1B\[[0-9]+m} [lindex $output 0] {} norm
    regsub -all {\x1B\[[0-9]+m} [lindex $output 1] {} blue
    regsub -all {\x1B\[[0-9]+m} [lindex $output 2] {} red
    regsub -all {\x1B\[[0-9]+m} [lindex $output 3] {} pink
    $text insert end $norm
    $text insert end $blue
    $text insert end $red
    $text insert end $pink
    set end [$text index end]
    scan $end %d end
    if {$end > 1000} {
        set diff [expr $end - 1000]
        $text delete 1.0 $diff.0
    }        
    $text yview -pickplace end
}

proc g_cut {} {
    g_copy
    set own [selection own]
    catch {$own delete sel.first sel.last}
}

proc g_copy {} {
    if ![catch {selection get} sel] {
        clipboard clear
        clipboard append $sel
    }
}

proc g_paste {} {
    global g_lastFocus
    if ![catch {selection get -selection CLIPBOARD} sel] {
        set widget [focus]
        if [catch {$widget insert insert $sel}] {
            catch {$g_lastFocus insert insert $sel}
        }    
    }
}

proc g_webNews {} {
    global g_wwwserver g_wwwsuffix
    set html [g_tcpip $g_wwwserver 80 "GET /OpenGENIE$g_wwwsuffix/news.htm\r\n"]
    g_help 0 $html
}

proc g_webFAQ {} {
    global g_wwwserver g_wwwsuffix
    set html [g_tcpip $g_wwwserver 80 "GET /OpenGENIE$g_wwwsuffix/faq.htm\r\n"]
    g_help 0 $html
}

proc g_webSub {} {
    set w .web
    g_winPosition $w
    wm title $w "Subsribe to mailing list"
    tixLabelEntry $w.name -label "Name:"
    $w.name subwidget entry config -textvariable gw_name
    $w.name subwidget entry delete 0 end
    tixLabelEntry $w.email -label "Email address:"
    $w.email subwidget entry config -textvariable gw_email
    $w.email subwidget entry delete 0 end  
    tixButtonBox $w.box -orientation horizontal
    $w.box add ok -text Ok -command "g_webSubOk; catch {destroy $w}"
    $w.box add help -text Help -command {g_help mailing.htm}
    $w.box add cancel -text Cancel -command "catch {destroy $w}"
    pack $w.name $w.email $w.box -fill x
    bind $w <Return> "$w.box.ok invoke"
    focus [$w.name subwidget entry]
}

proc g_webSubOk {} {
    global gw_name gw_email g_pd g_cgiserver
    set list [g_httpPost genie-announce]
    set name [g_httpPost $gw_name]
    set email [g_httpPost $gw_email]
    set data "ListName=$list&UserName=$name&UserEmail=$email"
    set len [string length $data]
    set m "POST /cgi-bin/ml_subscribe HTTP/1.0" 
    set h1 "From: $gw_email"
    set h2 "UserAgent: OpenGENIE"
    set h3 "Content-Type: application/x-www-form-urlencoded"
    set h4 "Content-Length: $len"
#g_tcpip $g_cgiserver 80 "$m\n$h1\n$h2\n$h3\n\n$data"
    g_tcpip $g_cgiserver 80 "$m\r\n$h1\r\n$h2\r\n$h3\r\n$h4\r\n\r\n$data"
    set g_pd(name) $gw_name
    set g_pd(email) $gw_email
}

proc g_webUnsub {} {
    global g_pd
    set w .web
    g_winPosition $w
    wm title $w "Unsubsribe from the mailing list"
    tixLabelEntry $w.name -label "Name:"
    $w.name subwidget entry delete 0 end
    $w.name subwidget entry insert 0 $g_pd(name)
    tixLabelEntry $w.email -label "Email address:"
    $w.email subwidget entry config -textvariable gw_email
    $w.email subwidget entry delete 0 end  
    $w.email subwidget entry insert 0 $g_pd(email)
    tixButtonBox $w.box -orientation horizontal
    $w.box add ok -text Ok -command "g_webUnsubOk; catch {destroy $w}"
    $w.box add help -text Help -command {g_help mailing.htm}
    $w.box add cancel -text Cancel -command "catch {destroy $w}"
    pack $w.name $w.email $w.box -fill x
    bind $w <Return> "$w.box.ok invoke"
    focus [$w.name subwidget entry]
}

proc g_webUnsubOk {} {
    global gw_email g_pd g_cgiserver
    set list [g_httpPost genie-announce]
    set email [g_httpPost $gw_email]
    set data "ListName=$list&UserEmail=$email"
    set len [string length $data]
    set m "POST /cgi-bin/ml_unsubscribe HTTP/1.0" 
    set h1 "From: $gw_email"
    set h2 "UserAgent: OpenGENIE"
    set h3 "Content-Type: application/x-www-form-urlencoded"
    set h4 "Content-Length: $len"
    g_tcpip $g_cgiserver 80 "$m\r\n$h1\r\n$h2\r\n$h3\r\n$h4\r\n\r\n$data"
    set g_pd(name) NONE
    set g_pd(email) NONE
}

proc g_webReport {} {
    global g_pd
    set w .web
    g_winPosition $w
    wm title $w "Report a bug"
    tixLabelEntry $w.subject -label "Subject:"
    $w.subject subwidget entry config -textvariable gw_subject
    $w.subject subwidget entry delete 0 end
    tixLabelEntry $w.from -label "From:"
    $w.from subwidget entry config -textvariable gw_from
    $w.from subwidget entry delete 0 end  
    $w.from subwidget entry insert 0 "$g_pd(name) <$g_pd(email)>"
    text $w.bug -height 10
    tixButtonBox $w.box -orientation horizontal
    $w.box add ok -text Ok -command "g_webReportOk; catch {destroy $w}"
    $w.box add help -text Help -command {g_help web.htm}
    $w.box add cancel -text Cancel -command "catch {destroy $w}"
    pack $w.subject $w.from -fill x
    pack $w.bug -fill both -expand 1
    pack $w.box -fill x
    bind $w <Return> "$w.box.ok invoke"
    focus [$w.subject subwidget entry]
}

proc g_webReportOk {} {
    global gw_subject gw_from
}

proc g_about {} {
    global g_tkDir
    set im [image create photo -file $g_tkDir/splash.gif]
    set text [gcl "version" -out]
    set o .output
    g_winPosition $o
    wm title $o "About"
    $o config -bg white
    label $o.gif -image $im
    text $o.main -height 15 -width 70 -bg white -relief flat -borderwidth 0
    button $o.button -text Ok -command "catch {destroy $o}" -bg white
    pack $o.gif
    pack $o.main -fill both -expand true
    $o.main insert end "\n$text\n"
    $o.main insert end "Open GENIE, a program for neutron scattering data analysis & display\n"
    $o.main insert end "ISIS facility, Rutherford Appleton Laboratory, Chilton, Didcot, OXON UK\n"
    $o.main insert end "Copyright (C) 1996-2000\n\n"
    $o.main insert end "Open GENIE comes with ABSOLUTELY NO WARRANTY; for details see `Warranty'.\n"
    $o.main insert end "This is free software, and you are welcome to redistribute it under certain\n"
    $o.main insert end "conditions; see `Licence' for details."
    pack $o.button
    $o.main config -state disabled
}
