proc Labels {instvars} {
  global labels font_options font_label colour_options colour_label
  set labels .alterLabels
  array set instarray $instvars
  catch {destroy $labels}
  toplevel $labels
  wm title $labels "Alter Labels"

  set colour $instarray(colour)
  set quality $instarray(quality)

  tixOptionMenu $labels.colour -label "Colour" -variable colour
  foreach opt $colour_options {
        $labels.colour add command $opt -label $colour_label($opt)
  }

  tixOptionMenu $labels.quality -label "Text Font" -variable quality
  foreach opt $font_options {
        $labels.quality add command $opt -label $font_label($opt)
  }
  $labels.colour config -value $colour
  $labels.quality config -value $quality

  tixLabelEntry $labels.size 	  -label "Text Size"       -labelside left
  tixLabelEntry $labels.thickness -label "Line Thickness"  -labelside left
  tixLabelEntry $labels.xlabel 	  -label "X Label" 	   -labelside left
  tixLabelEntry $labels.ylabel 	  -label "Y Label" 	   -labelside left

  $labels.size subwidget entry config -textvariable size
  $labels.thickness subwidget entry config -textvariable thickness
  $labels.xlabel subwidget entry config -textvariable xlabel
  $labels.ylabel subwidget entry config -textvariable ylabel

  tixButtonBox $labels.box -orientation horizontal
  $labels.box add ok     -text OK         -command "drawLabels {$instvars} "
  $labels.box add cancel -text Cancel     -command "catch {destroy $labels}"

  pack $labels.box $labels.size $labels.thickness $labels.ylabel \
       $labels.xlabel -side bottom -fill x -expand yes
  pack $labels.quality $labels.colour -side top -fill x -expand yes 

  frame $labels.top 
  frame $labels.bottom
  pack $labels.top $labels.bottom -side top -expand yes -fill x

  foreach type {xvert xhoriz} {
    radiobutton $labels.top.$type -text $type -variable xvert -value $type
    pack $labels.top.$type -side left -expand yes -fill x
  }

  foreach type {yvert yhoriz} {
    radiobutton $labels.bottom.$type -text $type -variable yvert -value $type
    pack $labels.bottom.$type -side left -expand yes -fill x
  }

  $labels.top.xhoriz select
  $labels.bottom.yvert select
}

proc drawLabels {instvars} {
  global colour xvert yvert quality xlabel ylabel labels size thickness
  array set instarray $instvars
  
  set size [string trim $size]
  if {[string len $size] == 0} {
    set size $instarray(charsize)
  }

  set thickness [string trim $thickness]
  if {[string len $thickness] == 0} {
    set thickness $instarray(lineThickness)
  }

  set xlabel [string trim $xlabel]
  if {[string len $xlabel] == 0} {
    set xlabel $instarray(xlabel)
  }

  set ylabel [string trim $ylabel]
  if {[string len $ylabel] == 0} {
    set ylabel $instarray(ylabel)
  }

  if {[string match $xvert xvert] == 1} {
    set xvert 1
  } else {
    set xvert 0
  }

  if {[string match $yvert yvert] == 1} {
    set yvert 1
  } else {
    set yvert 0
  }

  set objectid $instarray(objid)
  set tmp [string trim $objectid {] Array (2)} ]
  set tmp [string trim $tmp {[} ]
  set num [string wordend $tmp 0]
  set object [string range $tmp 0 $num]
  set picture [expr [string range $tmp [expr $num + 1] [string len $tmp] ] + 1]

  set output [smalltalk "|tmp temp|
	     tmp _ ((gXUUgraphics at: #gXcurrentgraphics) alterTCL: $object
		     and: $picture with: '$objectid' ).
	     (tmp isNil)
	     ifFalse:\[temp _ (tmp xlabel: '$xlabel'; ylabel: '$ylabel'; 
			      xvert: $xvert; yvert: $yvert; charsize: $size;
			      lineThickness: $thickness; colour: $colour;
			      quality: $quality; create).
		       (gXUUgraphics at: #gXcurrentdevice) Clear.
		((gXUUgraphics at: #gXcurrentgraphics) drawPicture: $picture) \]
	     ifTrue:\['Labels has not been altered' prinl\] !"]
	 
  testOutput $output
  destroy $labels
}
