proc exAxes {which} {
  set output [gcl "draw/$which/noxnum/noynum"]
  testOutput $output
}

proc exLine {mark} {
  global lcolour lwidth tsize tcolour colour_options colour_label symbol_label \
         symbol_options

  set line .lineGraph
  catch {destroy $line}
  toplevel $line
  wm title $line "Line/Errors/Markers Options"

  tixLabelEntry $line.one -label "Text Size " -labelside left
  $line.one subwidget entry config -textvariable tsize

  tixOptionMenu $line.colour -label "Line Colour"   -variable colour 
  foreach opt $colour_options {
	$line.colour add command $opt -label $colour_label($opt)
  }

  tixOptionMenu $line.tcolour -label "Text Colour"   -variable tcolour 
  foreach opt $colour_options {
	$line.tcolour add command $opt -label $colour_label($opt)
  }

  if {[string match $mark 3] == 1} {
    tixOptionMenu $line.symbol -label "Marker Symbol" -variable symbol
    foreach opt $symbol_options {
  	  $line.symbol add command $opt -label $symbol_label($opt)
    }
    pack $line.symbol -expand yes -fill x
  }

  tixButtonBox $line.box -orientation horizontal
    $line.box add ok -text OK -underline 0 -command "displayLine {$mark}" \
    -width 10
    $line.box add cancel -text Cancel -underline 0 -command "destroy $line" \
    -width 10
 
  pack $line.one $line.tcolour $line.colour $line.box -side top -expand yes \
       -fill x
}

proc displayLine {line} {
  global lcolour tcolour tsize symbol
  
  if {[string match $line 1] == 1} {
    set output [gcl "display/line _ _ _ _ _ _ _ _ _ _ _ _ _ _ $lcolour _ $tcolour $tsize"]
    testOutput $output
  } 
  if {[string match $line 2] == 1} {
     set output [gcl "display/errors _ _ _ _ _ _ _ _ _ _ _ _ _ _ $lcolour _ $tcolour $tsize"]
    testOutput $output
  } 
  if {[string match $line 3] == 1} {
    gcl "alter/markers $symbol"
    set output [gcl "display/markers _ _ _ _ _ _ _ _ _ _ _ _ _ _ $lcolour _ $tcolour $tsize"]
    testOutput $output
  }  

} 

proc limits {} {
  global xmin xmax ymin ymax
  
  set lim .disLimits
  catch {destroy $lim}
  toplevel $lim
  wm title $lim Limits

  tixLabelEntry $lim.one -label "XMin" -labelside left
  tixLabelEntry $lim.two -label "XMax" -labelside left
  tixLabelEntry $lim.three -label "YMin" -labelside left
  tixLabelEntry $lim.four -label "YMax " -labelside left

  $lim.one subwidget entry config -textvariable xmin
  $lim.two subwidget entry config -textvariable xmax 
  $lim.three subwidget entry config -textvariable ymin
  $lim.four subwidget entry config -textvariable ymax

  tixButtonBox $lim.box -orientation horizontal
    $lim.box add ok -text OK -command limitsTest -width 10
    $lim.box add cancel -text Cancel -command "catch {destroy $lim}" \
    -width 10
 
  pack $lim.one $lim.two $lim.three $lim.four -side top
  pack $lim.box -side bottom -fill x
}
   
proc limitsTest {} {
  global xmin xmax ymin ymax lim

  if {([string match *.* $xmin]==1) && ([string match *.* $xmax]==1) \
      && ([string match *.* $ymin]==0) && ([string match *.* $ymin]==0)} {

    set output [gcl "limits/no_auto"]
    testOutput $output
    gcl "display"
  }

  if {([string match *.* $xmin]==1) && ([string match *.* $xmax]==1) \
      && ([string match *.* $ymin]==0) && ([string match *.* $ymin]==0)} {

    set output [gcl "limits/x $xmin $xmax"]
    testOutput $output
    gcl "display"
  }

  if {([string match *.* $xmin]==0) && ([string match *.* $xmax]==0) \
      && ([string match *.* $ymin]==1) && ([string match *.* $ymin]==1)} {

    set output [gcl "limits/y $ymin $ymax"]
    testOutput $output
    gcl "display"
  }
  destroy $lim
}

#proc hardcopy {} {
#  global print 
#  
#  set print .hardcopy
#  catch {destroy $print}
#  toplevel $print
#  wm title $print "Print Options"
#
#  tixLabelEntry $print.file -label "File Name" -labelside left
#  tixLabelEntry $print.printer -label "Printer Name" -labelside left
##
#  $print.file subwidget entry config -textvariable file
#  $print.printer subwidget entry config -textvariable printer
#
#  tixControl $print.copies -label "Number of Copies" -integer true \
##	-variable copies -min 1 -max 999
#
#  tixButtonBox $print.box -orientation horizontal
#    $print.box add ok -text Print -command ok -width 6
#    $print.box add cancel -text Cancel -command "destroy $print" -width 6
#
#  pack $print.file $print.printer -side top -fill x
#  pack $print.box -side bottom -fill x
#
#  frame $print.top
#  frame $print.middle
#  frame $print.bottom
#  pack $print.top $print.middle $print.bottom $print.copies -side top \
#       -expand yes -fill x
# 
#  label $print.top.label -text "Save To: " 
#  pack $print.top.label -side left
#  foreach type {file printer both} {
#    radiobutton $print.top.$type -text $type -variable copy -value $type
#    pack $print.top.$type -side left
#  }
#  
#  foreach type {colour b/w} {
#    radiobutton $print.middle.$type -text $type -variable colour -value $type
#    pack $print.middle.$type -side right
#  }
#
#  foreach type {landscape portrait} {
#    radiobutton $print.bottom.$type -text $type -variable orientation \
#		-value $type
#    pack $print.bottom.$type -side right
#  }
#
#  $print.top.both select
#  $print.middle.colour select
#  $print.bottom.portrait select
#  
#}
 
#proc ok {} {
#  global file print printer copies orientation colour copy
#
#
#  if {[string match $colour colour] == 1} {
#    set colour c 
#  } else {
#    set colour ""
##  }
#
#  if {[string match $orientation portrait] == 1} {  	
#    set orientation v
#  } else {
#    set orientation ""
#  }
#  set ps ps
#  set method $orientation$colour$ps
#
#  set output [gcl "hardcopy \"$file\" \"$method\""]
#  testOutput $output
#  if {[string match $copy printer] == 1 || [string match $copy both] == 1} {
#    set printer [string trimleft $printer lpr]
#    set printer [string trim $printer ]
#
#    for {set i 0} {$i < $copies} {incr i 1} {
## NEED TO CHANGE THIS LINE, SO THEN IT CAN COPE WITH ALL TYPES
## OF COMPUTERS
#      exec "lpr" $printer $file
#    }
#  }
#  destroy $print
#}
#
