#include "genie_data_access.h"

#if defined(__ultrix) || defined(_WIN32)
#include "getopt.h"
//extern "C" int getopt(int argc, char **argv, char *optstring);  // missing prototype
#endif /* __ultrix */


void load_some_isis_data(const char *file)
{
    cout << "Load a little ISIS data" << endl;
    GX_select_source(file);
    
    GX_get("H1", "NSP1", 0);    // total no of spectra
    GX_get("H2", "", 3);        // a complete spectrum workspace
    return;
}

void save_some_isis_data(const char *file)
{
    cout << "Save the handles directly into an intermediate file" << endl;

    GX_select_destination(file, 0);
    GX_put("H1", "Nsp1", 0, "new", "Test data points");
    GX_put("H2", "data", 0, "", "Test data");
    return;
}

void write_some_test_data(const char *file, const char *output_format)
{
    cout << "Save some test data into isis.nxs" << endl;
    GX_select_destination(file, output_format);

    // Open a new file and write some test data to it
    char    s[]     = "A single string";
    int     a       = 123;
    long    b       = 1234;
    float   c       = 123.4F;
    double  d       = 1234.5;
    char    ss[]    = "one stringtwo string";
    int     aa[2]   = { 123, 456 };
    long    bb[2]   = { 1234, 5678 };
    float   cc[2]   = { 123.4F, 567.8F };
    double  dd[2]   = { 1234.5, 6789.1 };

    int ndims = 1;
    GX_transfer("S1", "<--", "STRING", s, &ndims, CDims(strlen(s)));
    GX_transfer("I1", "<--", "INT32", &a, 0, 0 );
    GX_transfer("I2", "<--", "INT64", &b, 0, 0 );
    GX_transfer("R1", "<--", "FLOAT32", &c, 0, 0 );
    GX_transfer("R2", "<--", "FLOAT64", &d, 0, 0 );

    GX_put("S1", "data", 0, "NEW", "Test string");              // First item into file
    GX_put("I1", "data", 0, "", "Test int32");
    GX_put("I2", "data", 0, "", "Test int64");
    GX_put("R1", "data", 0, "", "Test float32");
    GX_put("R2", "data", 0, "", "Test float64");

    GX_transfer("SS1", "<--", "STRING[]", ss, &ndims, CDims(10) );
    GX_transfer("II1", "<--", "INT32[]", aa, &ndims, CDims(2) );
    GX_transfer("II2", "<--", "INT64[]", bb, &ndims, CDims(2) );
    GX_transfer("RR1", "<--", "FLOAT32[]", cc, &ndims, CDims(2) );
    GX_transfer("RR2", "<--", "FLOAT64[]", dd, &ndims, CDims(2) );

    GX_put("SS1", "data", 0, "", "Test string");
    GX_put("II1", "data", 0, "", "Test int32");
    GX_put("II2", "data", 0, "", "Test int64");
    GX_put("RR1", "data", 0, "", "Test float32");
    GX_put("RR2", "data", 0, "", "Test float64");
    return;
}

void read_some_test_data()
{
    char    s[256];
    //int     a;
    long    b;
    float   c;
    double  d;
    //char    ss[1024];
    //int     aa[2];
    long    bb[2];
    //float   cc[2];
    //double  dd[2];
    cout << "Read some test data from isis.int" << endl;

    int ndims = 1;
    GX_select_source("isis.in3");
    GX_get("V1", "", 2);            // Integer
    GX_get("V2", "DATA", 0);        // String
    GX_get("VV", "", 4);            // Real
    GX_transfer("V1", "-->", "INT64", &b, 0, 0 );
    GX_transfer("V2", "-->", "STRING", s, &ndims, CDims(256) );
    GX_transfer("VV", "-->", "FLOAT64", &d, 0, 0 );
    GX_transfer("VV", "-->", "FLOAT32", &c, 0, 0 );
    GX_assign_handle("_a", "printn(V1,\" \",V2,\" \",VV)");
    cout << "Values read" << endl;
    cout << "b = " << b << endl;
    cout << "s = " << s << endl;
    cout << "d = " << d << endl;
    cout << "c = " << c << endl;

    GX_select_source("../bigfiles/hrp08639.raw");
    GX_get("VV", "SPEC", 0);            // Integer array
    GX_assign_handle("_a", "printn(VV)");
    GX_transfer("VV", "-->", "INT64*", bb, &ndims, CDims(2) );
    cout << bb[0] << " " << bb[1] << " duff->> " << bb[2] << endl;
    return;
}


void do_a_dir(const char *filename)
{
    cout << "Directory of \"" << filename << "\"" << endl
         << "=======================" << endl;
    GX_directory(filename);

    return;
}

main(int argc, char **argv)
{
    cout << "Opening Open GDAI session..." << endl;

    int debugging_output;
    if (getopt(argc, argv, "gsydhBEc") == 'c')
        debugging_output = 1;
    else
        debugging_output = 0;

    GX_activate_session("GENIE", 0, debugging_output);          // Specifies default output format

    load_some_isis_data("../bigfiles/hrp08639.raw");  // from a raw file

    GX_assign_handle("H2.NEW_FIELD", "H1");                     // add a new field to the handle

    save_some_isis_data("isis.raw");                            // to a new intermediate file

    GX_release_handle("H1");                        // Free up storage
    GX_release_handle("H2");

    // Have look at the intermediate file
    write_some_test_data("isis.nxs", "NeXus");               // as HDF
    write_some_test_data("isis.in3", "GENIE");             // as GENIE intermediate

    // inspect what we have produced
    do_a_dir("isis.raw");
    do_a_dir("isis.in3");
    do_a_dir("isis.nxs");

    read_some_test_data();

    cout << "Closing Open GDAI session" << endl;
    GX_deactivate_session();

    return 0;
}
