C
C Function for contour (array position -> world coordinates)
C
!MS$ATTRIBUTES DLLEXPORT :: MYCONTFUNC
        SUBROUTINE MYCONTFUNC(FLAGS, VAL_IN, VAL_OUT)
        IMPLICIT NONE
        INTEGER FLAGS
        DOUBLE PRECISION VAL_IN(*), VAL_OUT(*)
        REAL TR(6), INVTR(6)
        COMMON /TRMAT/ TR,INVTR
        SAVE /TRMAT/
        VAL_OUT(1)=TR(1) + VAL_IN(1) * TR(2) + VAL_IN(2) * TR(3)
        VAL_OUT(2)=TR(4) + VAL_IN(1) * TR(5) + VAL_IN(2) * TR(6)
        RETURN
        END
C
C For cell_array (world coordinates -> array position)
C
!MS$ATTRIBUTES DLLEXPORT :: INV_MYCONTFUNC
        SUBROUTINE INV_MYCONTFUNC(FLAGS, VAL_IN, VAL_OUT)
        IMPLICIT NONE
        INTEGER FLAGS
        DOUBLE PRECISION VAL_IN(*), VAL_OUT(*)
        REAL TR(6), INVTR(6)
        COMMON /TRMAT/ TR,INVTR
        SAVE /TRMAT/
        VAL_OUT(1)=INVTR(1) + VAL_IN(1) * INVTR(2) + 
     1             VAL_IN(2) * INVTR(3)
        VAL_OUT(2)=INVTR(4) + VAL_IN(1) * INVTR(5) + 
     1             VAL_IN(2) * INVTR(6)
        RETURN
        END
C
C Set up parameters for functions
C
!MS$ATTRIBUTES DLLEXPORT :: MYCONTSETUP
      SUBROUTINE MYCONTSETUP(PARS_GET,PARS_PUT)
      IMPLICIT NONE
      INTEGER   MXI, MXJ
      PARAMETER (MXI=40, MXJ=40)
      INCLUDE 'genie_modules.inc'
      INTEGER I,J,DIMS_ARRAY(2)
      REAL F(MXI,MXJ),FMIN,FMAX,ALEV,CONT(21),X(MXI),Y(MXJ)
      REAL INVTR(6), TR(6), ANGLE, RADIUS, DET
      COMMON /TRMAT/ TR,INVTR
      SAVE /TRMAT/
      EXTERNAL PARS_GET,PARS_PUT
C *** check library version
      IF (MODULE_VERSION_OK(GENIE_MAJOR, GENIE_MINOR) .EQ. 0) RETURN
C
C Compute a suitable function.
C Similar to one used in PGPLOT demonstration 4
C
      FMIN = 0.0
      FMAX = 0.0
      DO 20 I=1,MXI
          DO 10 J=1,MXJ
C              F(I,J) = COS(0.3*SQRT(I*2.)-0.4*J/3.)*COS(0.4*I/3)+
C     1                     (I-J)/40.0
              F(I,J) = COS(0.6*SQRT(I*80./MXI)-16.0*J/(3.*MXJ))*
     :                 COS(16.0*I/(3.*MXI))+(I/FLOAT(MXI)-J/FLOAT(MXJ))
              FMIN = MIN(F(I,J),FMIN)
              FMAX = MAX(F(I,J),FMAX)
   10     CONTINUE
   20 CONTINUE
C Contour levels
      DO 30 I=1,21
          CONT(I) = FMIN + (I-1)*(FMAX-FMIN)/20.0
   30 CONTINUE
C X and Y values: -40 to 40 
      DO I=1,MXI
        X(I) = -40. + FLOAT(I-1) / FLOAT(MXI-1) * 80.
      ENDDO
      DO J=1,MXJ
        Y(J) = -40. + FLOAT(J-1) / FLOAT(MXJ-1) * 80.
      ENDDO
C Transform matrix
C
C Compute the coordinate transformation matrix.
C ANGLE is the image rotation angle in radians.
C
      ANGLE=-120./57.29578
      RADIUS= 40.*SQRT(2.)
      TR(1) =-(40./MXI)*COS(ANGLE)+(40./MXJ)*SIN(ANGLE)-
     :         RADIUS*COS(ANGLE+45/57.29578)
      TR(2) = 80.*COS(ANGLE)/MXI
      TR(3) =-80.*SIN(ANGLE)/MXJ
      TR(4) =-(40./MXI)*SIN(ANGLE)-(40./MXJ)*COS(ANGLE)-
     :         RADIUS*SIN(ANGLE+45/57.29578)
      TR(5) = 80.*SIN(ANGLE)/MXI
      TR(6) = 80.*COS(ANGLE)/MXJ
C
C *** compute reciprocal transform (i.e. invert the matrix)
C
      DET = TR(2)*TR(6) - TR(3)*TR(5)
      INVTR(1) = (TR(3)*TR(4) - TR(6)*TR(1)) / DET
      INVTR(2) = TR(6) / DET
      INVTR(3) = -TR(3) / DET
      INVTR(4) = (TR(5)*TR(1) - TR(2)*TR(4)) / DET
      INVTR(5) = -TR(5) / DET
      INVTR(6) = TR(2) / DET
C
      CALL MODULE_PUT_REAL_ARRAY(PARS_PUT, 'TR', TR, 6)
      CALL MODULE_PUT_REAL_ARRAY(PARS_PUT, 'INVTR', INVTR, 6)
      CALL MODULE_PUT_REAL_ARRAY(PARS_PUT, 'C', CONT, 21)
      CALL MODULE_PUT_REAL_ARRAY(PARS_PUT, 'X', X, MXI)
      CALL MODULE_PUT_REAL_ARRAY(PARS_PUT, 'Y', Y, MXJ)
      DIMS_ARRAY(1) = MXI
      DIMS_ARRAY(2) = MXJ
      CALL MODULE_PUT_ND_REAL_ARRAY(PARS_PUT, 'V', F, DIMS_ARRAY, 2)
      RETURN
      END
C
C *** used by arc.gcl to generate points
C
!MS$ATTRIBUTES DLLEXPORT :: ARCGEN
      SUBROUTINE ARCGEN(PARS_GET,PARS_PUT)
      IMPLICIT NONE
      INTEGER I, N, N2
      REAL STEP, T, X1, Y1, X2, Y2, PI
      PARAMETER(N2=50, N=2*N2, PI=3.14159)
      REAL XP(N), YP(N), X, Y, R1, R2, THETA1, THETA2 
      INCLUDE 'genie_modules.inc'
      EXTERNAL PARS_GET,PARS_PUT
C *** check library version
      IF (MODULE_VERSION_OK(GENIE_MAJOR, GENIE_MINOR) .EQ. 0) RETURN
      CALL MODULE_GET_REAL(PARS_GET, 'X', X)
      CALL MODULE_GET_REAL(PARS_GET, 'Y', Y)
      CALL MODULE_GET_REAL(PARS_GET, 'R1', R1)
      CALL MODULE_GET_REAL(PARS_GET, 'R2', R2)
      CALL MODULE_GET_REAL(PARS_GET, 'THETA1', THETA1)
      CALL MODULE_GET_REAL(PARS_GET, 'THETA2', THETA2)
      THETA1=THETA1*PI/180.
      THETA2=THETA2*PI/180.
      STEP=(THETA1-THETA2)/FLOAT(N2-1)
      DO I=1,N2
          T=THETA2+STEP*FLOAT(I-1)
          X1=X+R1*COS(T)
          Y1=Y+R1*SIN(T)
          X2=X+R2*COS(T)
          Y2=Y+R2*SIN(T)
          XP(I)=X1
          YP(I)=Y1
          XP(N-I+1)=X2
          YP(N-I+1)=Y2
      ENDDO
      CALL MODULE_PUT_REAL_ARRAY(PARS_PUT, 'X', XP, N)
      CALL MODULE_PUT_REAL_ARRAY(PARS_PUT, 'Y', YP, N)
      END
C
